angular.module('easyscott.acn', [])
  .controller('ACNController', [ '$scope', '$rootScope', 'WebsocketService', 'TimecodeService', 'DeviceModel', 'ChannelModel', 'CjamSettingsService', 'SystemPropertiesModel', function($scope, $rootScope, WebsocketService, TimecodeService, DeviceModel, channelModel, CjamSettingsService, SystemPropertiesModel) {

    // For easy layer-breaking access from the template
    $scope.cjam = CjamSettingsService;

    $scope.status = {
      framerate: {},
      projectFramerate: {},
      channel: {},
      cjamRunning: false,
      timecodeSet: false,
      userbits: {
        one: '00',
        two: '00',
        three: '00',
        four: '00'
      },
      tunePwm: ''
    };
    $scope.editables = {
      framerate: null,
      projectFramerate: null,
      channels: null,
      userbits: null,
      tunePwm: null
    };
    $scope.timecode = {
      hours: null,
      minutes: null,
      seconds: null,
      frames: null
    };
    $scope.isSetClientTimeAllowed = true;

    $scope.normalizeTC = function() {
      TimecodeService.normalizeTC($scope.timecode);
    };

    $scope.setTimecode = function() {
      var timecode = $scope.timecode.hours + ":" + $scope.timecode.minutes + ":" + $scope.timecode.seconds + ":" + $scope.timecode.frames;

      WebsocketService.sendMessage({
        request: 'CHANNEL_COMMAND',
        id: channelModel.getActiveACNChannelId(),
        command: 'SETTC',
        arguments: {
          tc: timecode
        }
      });
    };
    $scope.useClientTime = function() {
      var currentDate = new Date();
      $scope.timecode = TimecodeService.fromNow();
      $scope.normalizeTC();
    };
    $scope.setClientTime = function() {
      $scope.useClientTime();
      $scope.setTimecode();
      $scope.resetTimecode();
      $scope.isSetClientTimeAllowed = false;

      var locale = SystemPropertiesModel.properties['useEuropeanUserbits'] === 'true' ? 'de' : 'en';
      $scope.setClientDateToUserbits(locale);
      $scope.updateUserbits();

      if (!$scope.generator) {
        setTimeout(function () {
          CjamSettingsService.setCjamMaster(true);
        }, 200);
      }
    };
    $scope.resetTimecode = function() {
      $scope.timecode = {
        hours: null,
        minutes: null,
        seconds: null,
        frames: null
      };
    };
    $scope.updateFramerate = function() {
      CjamSettingsService.setProjectFramerate($scope.editables.projectFramerate.value);
      $scope.status.projectFramerate = $scope.editables.projectFramerate;
      $scope.editables.projectFramerate = null;
    };
    $scope.updateChannel = function() {
      CjamSettingsService.setChannel($scope.editables.channel.value);
      $scope.status.channel = $scope.editables.channel;
      $scope.editables.channel = null;
    };
    $scope.setClientDateToUserbits = function(locale) {
      if (!locale) {
        locale = "en";
      }
      var currentDate = new Date();

      var dayOfMonth = currentDate.getDate();
      if(dayOfMonth < 10) {
        dayOfMonth = "0" + dayOfMonth;
      }
      var month = currentDate.getMonth() + 1;
      if(month < 10) {
        month = "0" + month;
      }
      var year = currentDate.getYear() % 100;
      if(year < 10) {
        // ensure we are safe for the years 2100 -2109 ;-)
        year = "0" + year;
      }

      $scope.editables.userbits = {};
      if (locale == "de") {
        $scope.editables.userbits.one = dayOfMonth + "";
        $scope.editables.userbits.two = month + "";
        $scope.editables.userbits.four = "ED";
      } else {
        $scope.editables.userbits.one = month + "";
        $scope.editables.userbits.two = dayOfMonth + "";
        $scope.editables.userbits.four = "AD";
      }
      $scope.editables.userbits.three = year + "";
      $scope.updateUserbits();
    };
    $scope.updateUserbits = function() {
      var inputsAreValid = jQuery('input[name="userbit1"]').length == 0 || (jQuery('input[name="userbit1"]')[0].checkValidity() && jQuery('input[name="userbit2"]')[0].checkValidity() && jQuery('input[name="userbit3"]')[0].checkValidity() && jQuery('input[name="userbit4"]')[0].checkValidity());

      if(inputsAreValid) {
        var channelId = channelModel.getActiveACNChannelId();
        if (channelId) {
          var message = {
            request: 'CHANNEL_COMMAND',
            id: channelId,
            command: 'SET_USERBITS',
            arguments: {
              userbits: $scope.editables.userbits.one + $scope.editables.userbits.two + $scope.editables.userbits.three + $scope.editables.userbits.four
            }
          };

          WebsocketService.sendMessage(message);
        }
        $scope.status.userbits = $scope.editables.userbits;
        $scope.editables.userbits = null;
      }
    };
    $scope.incrementLastUserbit = function() {
      if($scope.editables.userbits.four < 99) {
        $scope.editables.userbits.four++;
        if($scope.editables.userbits.four < 10) {
          $scope.editables.userbits.four = "0" + $scope.editables.userbits.four;
        }
      }
    };
    $scope.decrementLastUserbit = function() {
      if($scope.editables.userbits.four > 0) {
        $scope.editables.userbits.four--;
        if($scope.editables.userbits.four < 10) {
          $scope.editables.userbits.four = "0" + $scope.editables.userbits.four;
        }
      }
    };
    $scope.updateSetClientTimeStatus = function() {
      const tc = $rootScope.currentTc();
      if (!tc) { return; }
      const todDelta = TimecodeService.difference(tc, TimecodeService.fromNow());
      $scope.isSetClientTimeAllowed = !$scope.status.timecodeSet || Math.abs(todDelta) > 5;
    };

    $scope.updateTunePwm = function() {
      var channelId = channelModel.getActiveACNChannelId();
      if (channelId) {
        var message = {
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_TUNE_PWM',
          arguments: {
            tunepwm: $scope.editables.tunePwm
          }
        }
      }
      WebsocketService.sendMessage(message);

      $scope.status.tunePwm = $scope.editables.tunePwm;
      $scope.editables.tunePwm = false;
    };

    CjamSettingsService.updateCjamStatus($scope.status);
    $scope.updateSetClientTimeStatus();

    DeviceModel.addEventListener('change', function(type) {
      CjamSettingsService.updateCjamStatus($scope.status);
      $scope.updateSetClientTimeStatus();
      $scope.$apply();
    });

    WebsocketService.addEventListener('message', function(type, message) {
      if(message.request == 'TIMECODE') {
        $scope.updateSetClientTimeStatus();
        $scope.$apply();
      }
    });

  }]);
