angular.module('easyscott.about', []).controller('AboutController', ['$scope', 'WebsocketService', 'SystemPropertiesModel', 'DeviceModel', 'ChannelModel', function ($scope, WebsocketService, SystemPropertiesModel, DeviceModel, ChannelModel) {
  $scope.logLevel = 'UNKNOWN';
  $scope.logFiles = [];
  $scope.logFilesChecked = { 'tool.log': true };
  $scope.showExtended = false;
  $scope.devSecret = "";
  $scope.serialPortTestStatus = {
  };
  $scope.lastInterval = new Date().valueOf();
  $scope.uploadRunning = false;
  $scope.uploadPercent = 0;

  $scope.enableDebugLogging = function () {
    WebsocketService.sendMessage({
      request: 'LOG_LEVEL',
      level: 'DEBUG'
    });
  };

  $scope.disableDebugLogging = function () {
    WebsocketService.sendMessage({
      request: 'LOG_LEVEL',
      level: 'INFO'
    });
  };

  $scope.enableDevMode = function () {
    WebsocketService.sendMessage({
      request: 'DEV_MODE',
      enabled: true,
      secret: $scope.devSecret
    });
  };

  $scope.disableDevMode = function () {
    WebsocketService.sendMessage({
      request: 'DEV_MODE',
      enabled: false,
      secret: $scope.devSecret
    });
  };

  $scope.requestLogFiles = function () {
    jQuery.ajax('/log/list')
      .success(function (fileList) {
        fileList.sort();
        $scope.logFiles = fileList;
        $scope.$apply();
      });
  };

  $scope.selectAllLogFiles = function () {
    $scope.logFiles.forEach(function (f) {
      $scope.logFilesChecked[f] = true;
    });
  };

  $scope.downloadLogFiles = function () {
    var checkedFiles = _.filter($scope.logFiles, function (f) {
      return $scope.logFilesChecked[f];
    });

    if (checkedFiles.length === 0) {
      return;
    }

    var fileList = checkedFiles.join(",");
    window.location = ('/log/download?fileList=' + fileList);
  };

  $scope.testSerialPort = function (port) {
    $scope.serialPortTestStatus[port] = "loading";

    jQuery.ajax('/serialPortTest/', {
      type: "POST",
      contentType: "text/plain; charset=UTF-8",
      processData: false,
      data: port,
      success: function () {
        $scope.serialPortTestStatus[port] = "success";
        $scope.$apply();
      },
      error: function () {
        $scope.serialPortTestStatus[port] = "fail";
        $scope.$apply();
      }
    });
  };

  $scope.applyCheatCode = function () {
    var message = {
      request: "UNLOCK_FEATURE",
      cheatCode: $scope.cheatCode
    };
    $scope.executeSynchronousRequest(message, function (result) {
      if (result && result.success) {
        alert("Unlock successful!");
      } else {
        alert("Unlock failed");
      }
    })
  };

  $scope.uploadManual = function (e) {
    e.preventDefault();

    if (!$scope.file) {
      $scope.clearAlerts();
      $scope.addAlert('danger', 'Please select a file to upload');
      return;
    }

    if (!$scope.file.name.match(/\.pdf$/)) {
      $scope.clearAlerts();
      $scope.addAlert('danger', 'The update file must end with ".pdf"');
      return;
    }

    $scope.uploadFile($scope.file);
  };

  $scope.uploadFile = function (file) {
    $scope.clearAlerts();
    $scope.uploadRunning = true;

    var data = new FormData();
    data.append("file", file);

    $.ajax({
      url: '/manual',
      type: 'POST',
      data: data,
      cache: false,
      processData: false,
      contentType: false,
      xhr: function () {
        var xhr = new window.XMLHttpRequest();
        xhr.upload.addEventListener('progress', $scope.updateProgress, false);
        return xhr;
      },
      success: function () {
        $scope.clearAlerts();
        $scope.addAlert('success', 'Manual successfully uploaded');
        $scope.uploadRunning = false;
        $scope.uploadPercent = 0;
      },
      error: function (xhr) {
        $scope.clearAlerts();
        $scope.addAlert('danger', 'An error occured while uploading the file: ' + xhr.responseText);
        $scope.uploadRunning = false;
        $scope.uploadPercent = 0;
      }
    });
  };

  $scope.updateProgress = function (event) {
    if (event.lengthComputable) {
      $scope.uploadPercent = parseInt(event.loaded / event.total * 100);
      $scope.$apply();
    }
  };

  $scope.deleteManual = function() {
    $.ajax({
      url: '/manual',
      type: 'DELETE',
      success: function () {
        $scope.clearAlerts();
        $scope.addAlert('success', 'Original manual restored');
      },
      error: function (xhr) {
        $scope.clearAlerts();
        $scope.addAlert('danger', 'An error occured while restoring the original manual');
      }
    });
  }

  $scope.isAutoPower = function() {
    var lockit = DeviceModel.getMasterclockDevice();
    if (lockit) {
      return parseInt(lockit.properties['autoPower']);
    } else {
      return 0;
    }
  };

  $scope.setAutoPower = function (power) {
    WebsocketService.sendMessage({
      request: 'CHANNEL_COMMAND',
      id: ChannelModel.getActiveACNChannelId(),
      command: 'SET_AUTO_POWER',
      arguments: {
        autoPower: power
      }
    });
  };

  DeviceModel.addEventListener('change', function() { $scope.$apply(); });

  WebsocketService.addEventListener('message', function (type, message) {
    if (message.request === 'LOG_LEVEL') {
      $scope.logLevel = message.logLevel.standardLevel;
      $scope.$apply();
    }
  });

  SystemPropertiesModel.addEventListener('change', function () {
    $scope.$apply();
  });

  WebsocketService.sendMessage({
    request: 'REQUEST_LOG_LEVEL'
  });

  $scope.requestLogFiles();

  $scope.ticker = setInterval(function () {
    var intervalDelta = new Date().valueOf() - $scope.lastInterval;
    $scope.lastInterval = new Date().valueOf();
    $("#about-tc").text($scope.currentTcString());
    $("#browser-offset").text(new Date().valueOf() - $scope.tc.reference);
    $("#browser-delay").text(intervalDelta);
  }, 0);

  $scope.$on('$destroy', function () {
    if ($scope.ticker) {
      clearInterval($scope.ticker);
    }
  });
}]);
