angular.module('easyscott.clink', [])
  .controller('CLinkController', [ '$rootScope', '$scope', 'WebsocketService', 'ChannelModel', 'DeviceModel', 'ClipModel', 'SystemPropertiesModel', function($rootScope, $scope, WebsocketService, channelModel, deviceModel, ClipModel, SystemPropertiesModel) {
    $scope.recording = false;
    $scope.loadingRecord = false;
    $scope.loadingTracks = false;
    $scope.loadingMeters = true;
    $scope.loadingDeviceData = false;
    $scope.loadingClips = false;
    $scope.loadingImport = false;
    $scope.loadingBulk = false;
    $scope.loadingTake = false;
    $scope.bulkLoadingFinished = false;
    $scope.showMeters = false;
    $scope.showTracks = false;
    $scope.showBulkImport = false;
    $scope.showHistory = false;
    $scope.clips = [];
    $scope.meters = [];
    $scope.batchSize = 100;
    $scope.clip = {
      current: {
        filename: null,
        takeId: null,
        scenelabel: null,
        takelabel: null,
        takeNumber: null,
        comment: null,
        tracks: []
      },
      next: {
        filename: null,
        takeId: null,
        scenelabel: null,
        takelabel: null,
        takeNumber: null,
        comment: null
      }
    };
    $scope.editTakeModel = {};
    $scope.tracks = [];
    $scope.clinkDevice = deviceModel.getCLinkDevice();
    $scope.hasSameTcClips = false;
    $scope.sysProperties = SystemPropertiesModel.properties;
    $scope.editables = {
        shouldApplySlateInfo: $scope.sysProperties.applySlateInfoToSoundRecorder === 'true'
    };

    WebsocketService.addEventListener('message', function(type, message) {
      if(message.request == 'CLINK_TAKE_LIST') {
        if (message.takes) {
          $scope.importClips(message.takes);
          $scope.$apply();
        }
      }
    });

    WebsocketService.addEventListener('message', function(type, message) {
      if(message.request == 'DEVICE_FUGITIVE_STATE') {
        if(message.state) {
          $scope.meters = message.state;
          $scope.loadingMeters = false;
          $scope.$apply();

          drawMetersInCanvas();
        }
      }
    });

    SystemPropertiesModel.addEventListener('change', function(type) {
        $scope.sysProperties = SystemPropertiesModel.properties;
        $scope.editables.shouldApplySlateInfo = $scope.sysProperties.applySlateInfoToSoundRecorder === 'true';
        $scope.$apply();
    });

    $scope.isReady = function() {
      return $scope.clinkDevice && $scope.clip.next.takelabel;
    };

    $scope.setBackendMeteringState = function(enabled) {
        var channelId = channelModel.getActiveCLinkChannelId();
        if(channelId && $scope.clinkDevice && $scope.clinkDevice.connected) {
            WebsocketService.sendMessage({
                request: 'CHANNEL_COMMAND',
                id: channelId,
                command: 'METER_MONITOR',
                arguments: {
                    enabled: enabled
                }
            });
        } else {
            setTimeout(function() { $scope.setBackendMeteringState(enabled); }, 100);
        }
    };

    $scope.enableMeterMonitor = function() {
      $scope.setBackendMeteringState(true);
      $scope.showMeters = true;
    };

    $scope.disableMeterMonitor = function() {
      $scope.showMeters = false;
      $scope.setBackendMeteringState(false);
    };

    $scope.toggleTracks = function() {
      if ($scope.showTracks) {
        $scope.showTracks = false;
        if ($scope.clinkDevice.properties.hasMeters !== "false") {
          $scope.disableMeterMonitor();
        }
      } else {
        $scope.showTracks = true;
        if ($scope.clinkDevice.properties.hasMeters !== "false") {
          $scope.enableMeterMonitor();
        }
      }
    };

    $scope.toggleMeterMonitor = function() {
        if ($scope.showMeters) {
            $scope.disableMeterMonitor();
        } else {
            $scope.enableMeterMonitor();
        }
    };

    $scope.tracksBusyIndicator = {
      show: function() {
        $scope.loadingTracks = true;
      },
      hide: function() {
        $scope.loadingTracks = false;
      }
    };

    $scope.deviceDataBusyIndicator = {
        show: function() {
          $scope.loadingDeviceData = true;
        },
        hide: function() {
          $scope.loadingDeviceData = false;
        }
    };

    $scope.clipsBusyIndicator = {
        show: function() {
          $scope.loadingClips = true;
        },
        hide: function() {
          $scope.loadingClips = false;
        }
    };

    $scope.recordBusyIndicator = {
        show: function() {
          $scope.loadingRecord = true;
        },
        hide: function() {
          $scope.loadingRecord = false;
        }
    };

    $scope.editDialogBusyIndicator = {
        show: function() {
          $scope.loadingTake = true;
        },
        hide: function() {
          $scope.loadingTake = false;
        }
    };

    $scope.refreshClips = function() {
      if($scope.showHistory) {
        var channelId = channelModel.getActiveCLinkChannelId();
        if(channelId) {
          $scope.clipsBusyIndicator.show();
          $scope.executeSynchronousRequest({
            request: 'CHANNEL_COMMAND',
            id: channelId,
            command: 'REQUEST_TAKENAMES'
          }, function(result) {
            if(result) {
              $scope.importClips(result.data);
            }
            $scope.clipsBusyIndicator.hide();
            $scope.$apply();
          });
        } else {
          setTimeout($scope.refreshClips, 100);
        }
      }
    };

    $scope.importClips = function(clipData) {
      var clips = _.chain(clipData).map(mapCLinkClip).value();
      $scope.clips = _.filter(clips, function(clip) {
        var clipsWithSameTc = _.filter(clips, function(candidate) {
          return candidate.tcStartSeconds === clip.tcStartSeconds && candidate.tcStartFrames === clip.tcStartFrames;
        }).length;
        return !(clip.tcStartSeconds === 0 && clip.tcStartFrames === 0) && clipsWithSameTc === 1;
      });


      if (clips.length !== $scope.clips.length) {
        console.log("diverge");
        console.log(clips);
        console.log($scope.clips);
      }

      $scope.hasSameTcClips = clips.length !== $scope.clips.length;
    };

    var updateClipSettings = function() {
      if($scope.clinkDevice) {
        $scope.clinkDevice.remoteControllable = $scope.clinkDevice.connected && $scope.clinkDevice.properties.hasRequiredFirmware;
        $scope.clinkDevice.clipsWithZeroTimecode = $scope.clinkDevice.connected && $scope.clinkDevice.properties.clipsWithZeroTimecode;

        // current clip
        $scope.clip.current.filename = $scope.clinkDevice.properties.currentFilename;
        $scope.clip.current.takeId = $scope.clinkDevice.properties.currentTakeIdentifier;
        $scope.clip.current.scenelabel = $scope.clinkDevice.properties.currentSceneLabel;
        $scope.clip.current.takelabel = $scope.clinkDevice.properties.currentTakeLabel;
        $scope.clip.current.takeNumber = parseInt($scope.clip.current.takelabel);
        $scope.clip.current.comment = $scope.clinkDevice.properties.currentNotes;
        $scope.clip.current.circle = $scope.clinkDevice.properties.currentCircled;
        $scope.clip.current.tracks = $scope.clinkDevice.properties.currentTracks;

        // next clip
        $scope.clip.next.filename = $scope.clinkDevice.properties.nextFilename;
        $scope.clip.next.takeId = $scope.clinkDevice.properties.nextTakeIdentifier;
        $scope.clip.next.scenelabel = $scope.clinkDevice.properties.nextSceneLabel;
        $scope.clip.next.takelabel = $scope.clinkDevice.properties.nextTakeLabel;
        $scope.clip.next.takeNumber = parseInt($scope.clip.next.takelabel);
        $scope.clip.next.comment = $scope.clinkDevice.properties.nextNotes;

        // recording
        $scope.recording = $scope.clinkDevice.properties.recording;

        // tracks
        var tracks = [];
        var trackTitles = $scope.clinkDevice.properties.trackTitles;
        var trackNames = $scope.clinkDevice.properties.trackNames;

        for (var i = 0; i < trackNames.length; i++) {
          tracks.push({
            index: i,
            title: trackTitles[i],
            name: trackNames[i]
          });
        }
        $scope.tracks = tracks;
      }
    };
    updateClipSettings();
    $scope.refreshClips();

    deviceModel.addEventListener('change', function(type) {
      $scope.clinkDevice = deviceModel.getCLinkDevice();
      updateClipSettings();

      if ($scope.clinkDevice && $scope.clinkDevice.properties && $scope.clinkDevice.properties.hasMeters === "false") {
        $scope.disableMeterMonitor();
      }

      $scope.$apply();
    });

    ClipModel.addEventListener('clipChanged', function(ev, type, clipId, clip) {
      if (type === 'sound') {

        // Update open edit popup
        if (jQuery('#dlgCLinkTake').css('display') !== 'none' && clipId === $scope.editTakeModel.id) {
          clip.takeNumber = parseInt(clip.takelabel);
          clip.takeId = $scope.editTakeModel.takeId;
          $scope.editTakeModel = clip;
          $scope.$apply();
        }
      }
    });

    $scope.editTake = function(clip, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        jQuery('#dlgCLinkTake').modal('show');
        busyIndicator.show();

        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'READ_TAKE',
          arguments: {
            takeId: clip.takeId
          }
        }, function(result) {
          if(result && result.data) {
            $scope.editTakeModel = _.chain(result.data).map(mapCLinkClip).head().value();
          }
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateScene = function(clip, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        busyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_SCENE_LABEL',
          arguments: {
            takeId: clip.takeId,
            value: clip.scenelabel
          }
        }, function(result) {
          if(result) {
            $scope.editTakeModel = _.chain(result.data).map(mapCLinkClip).head().value();
          }
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateTake = function(clip, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        busyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_TAKE_LABEL',
          arguments: {
            takeId: clip.takeId,
            value: adaptTakeNumber(clip.takeNumber, clip.takelabel)
          }
        }, function(result) {
          if(result) {
            $scope.editTakeModel = _.chain(result.data).map(mapCLinkClip).head().value();
          }
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateNotes = function(clip, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        busyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_NOTES',
          arguments: {
            takeId: clip.takeId,
            value: clip.comment
          }
        }, function(result) {
          if(result) {
            $scope.editTakeModel = _.chain(result.data).map(mapCLinkClip).head().value();
          }
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateCircle = function(clip, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        busyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_CIRCLE',
          arguments: {
            takeId: clip.takeId,
            value: JSON.stringify(clip.circle)
          }
        }, function(result) {
          if(result) {
            $scope.editTakeModel = _.chain(result.data).map(mapCLinkClip).head().value();
          }
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateClipTrackName = function(clip, track, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        busyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_CLIP_TRACK_NAME',
          arguments: {
            takeId: clip.takeId,
            trackIndex: track.number - 1,
            value: track.name
          }
        }, function(result) {
          if(result) {
            $scope.editTakeModel = _.chain(result.data).map(mapCLinkClip).head().value();
          }
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateTrackName = function(track, busyIndicator) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if (channelId) {
        busyIndicator.show();
        $scope.executeSynchronousRequest({
          request:'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_TRACK_NAME',
          arguments: {
            trackIndex: track.index,
            value: track.name
          }
        }, function(result) {
          busyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.falseTake = function(clip) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        $scope.deviceDataBusyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_FALSE_TAKE',
          arguments: {
            takeId: clip.takeId
          }
        }, function() {
          $scope.deviceDataBusyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.startRecording = function() {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId && $scope.clinkDevice && $scope.clinkDevice.id) {
        $scope.recordBusyIndicator.show();
        $scope.deviceDataBusyIndicator.show();
        $scope.tracksBusyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'RECORD_START',
          arguments: {
            device: $scope.clinkDevice.id
          }
        }, function() {
          $scope.recordBusyIndicator.hide();
          $scope.deviceDataBusyIndicator.hide();
          $scope.tracksBusyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.stopRecording = function() {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId && $scope.clinkDevice && $scope.clinkDevice.id) {
        $scope.recordBusyIndicator.show();
        $scope.deviceDataBusyIndicator.show();
        $scope.tracksBusyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'RECORD_STOP',
          arguments: {
            device: $scope.clinkDevice.id
          }
        }, function() {
          $scope.recordBusyIndicator.hide();
          $scope.deviceDataBusyIndicator.hide();
          $scope.tracksBusyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.toggleHistory = function() {
      if($scope.showHistory) {
        $scope.disableHistory();
      } else {
        $scope.enableHistory();
      }
    };
    $scope.enableHistory = function() {
      $scope.showHistory = true;
      $scope.refreshClips();
    };
    $scope.disableHistory = function() {
      $scope.showHistory = false;
    };

    $scope.toggleBulkImport = function() {
      if($scope.showBulkImport) {
        $scope.disableBulkImport();
      } else {
        $scope.enableBulkImport();
      }
    };
    $scope.enableBulkImport = function() {
      $scope.showBulkImport = true;
    };
    $scope.disableBulkImport = function() {
      $scope.showBulkImport = false;
    };

    $scope.isTrackArmed = function(index) {
      if ($scope.clinkDevice) {
        var armingBitmap = parseInt($scope.clinkDevice.properties.armingBitmap);
        return ((armingBitmap >> index) & 1) != 0;
      }

      return false;
    };

    $scope.bulkImport = function() {
      var channelId = channelModel.getActiveCLinkChannelId();
      if($scope.batchSize > 0 && channelId) {
        $scope.loadingBulk = true;
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'BULK_IMPORT',
          arguments: {
            size: $scope.batchSize
          }
        }, function() {
          $scope.loadingBulk = false;
          $scope.bulkLoadingFinished = true;
          $scope.$apply();
        });
      }
    };

    $scope.fullRefresh = function() {
      var channelId = channelModel.getActiveCLinkChannelId();
      if(channelId) {
        $scope.showHistory = true;
        $scope.clipsBusyIndicator.show();
        $scope.deviceDataBusyIndicator.show();
        $scope.tracksBusyIndicator.show();

        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'FULL_REFRESH'
        }, function(result) {
          $scope.clipsBusyIndicator.hide();
          $scope.deviceDataBusyIndicator.hide();
          $scope.tracksBusyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.switchArmState = function(trackIndex) {
      var channelId = channelModel.getActiveCLinkChannelId();
      if (channelId) {
        $scope.tracksBusyIndicator.show();
        $scope.executeSynchronousRequest({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_TRACK_ARMING',
          arguments: {
            trackIndex: trackIndex,
            value: !$scope.isTrackArmed(trackIndex)
          }
        }, function() {
          $scope.tracksBusyIndicator.hide();
          $scope.$apply();
        });
      }
    };

    $scope.updateSlateInfoAcceptance = function(event) {
        WebsocketService.sendMessage({
            request: 'SET_SYSTEM_PROPERTY',
            key: 'applySlateInfoToSoundRecorder',
            value: $scope.editables.shouldApplySlateInfo + ""
        });
    };

    var mapCLinkClip = function(item) {
      var clip = item.clip;
      clip.takeId = item.takeIdentifier.takeId;
      clip.takeNumber = parseInt(clip.takelabel);

      return clip;
    };

    var clipsEqual = function(c1, c2) {
      var tracks1 = _.map(c1.tracks, function(t) { return t.name; });
      var tracks2 = _.map(c2.tracks, function(t) { return t.name; });

      return c1.tcStartSeconds === c2.tcStartSeconds &&
        c1.tcStartFrames === c2.tcStartFrames &&
        c1.scenelabel === c2.scenelabel &&
        c1.takelabel == c2.takelabel &&
        c1.circle === c2.circle &&
        c1.comment === c2.comment &&
        _.isEqual(tracks1, tracks2);
    };

    // Adds leading zeros to the given number until it matches
    // the length of the original string (e.g. a take number string).
    var adaptTakeNumber = function(n, originalString) {
      var r = n.toString();
      while (r.length < originalString.length) {
        r = "0" + r;
      }
      return r;
    };

    // AUDIOMETERS

    var drawMetersInCanvas = function() {
      for (var i = 0; i < $scope.meters.length; i++) {
        var $element = $('#meter-' + i);
        var element = $element[0];
        if (!element) {
          continue;
        }

        element.width = $element.parent().width();

        var context = element.getContext('2d');
        if (!context) {
          continue;
        }

        var vuFraction = Math.max(0, (100 - $scope.meters[i].vu * 2.5) / 100);
        var peakFraction = (100 - $scope.meters[i].peak * 2.5) / 100;

        if ($scope.meters[i].vu <= 0) {
          vuFraction = 0;
        }

        context.clearRect(0, 0, element.width, element.height);

        context.fillStyle = '#00ff00';
        context.fillRect(0, 0, element.width * vuFraction, element.height);

        context.fillStyle = '#ff0000';
        context.fillRect(element.width * peakFraction, 0, 2, element.height);
      }
    };

    // Stop metering by default
    $scope.disableMeterMonitor();

    // Ensure backend metering state.
    // This is needed when another web client disabled metering, for example.
    $rootScope.ensureMeterTimer = setInterval(function() {
      if ($scope.showMeters) {
        $scope.setBackendMeteringState(true);
      }
    }, 1000);


    /*
     ============================ FIX FOR MOBILE SAFARI ==============================
     Modified version of the workaround described in this blog post:
     http://www.abeautifulsite.net/bootstrap-3-modals-and-the-ios-virtual-keyboard/

     This works around a bug in mobile Safari described here:
     https://github.com/twbs/bootstrap/issues/9023
     */

    // iOS check...ugly but necessary
    if( navigator.userAgent.match(/iPhone|iPad|iPod/i) ) {
      $('.modal').on('show.bs.modal', function() {
        // Position modal absolute and bump it down to the scrollPosition
        $(this)
          .css({
            position: 'absolute',
            marginTop: $(window).scrollTop() + 'px',
            bottom: 'auto'
          });

        // Position backdrop absolute and make it span the entire page
        //
        // Also dirty, but we need to tap into the backdrop after Boostrap 
        // positions it but before transitions finish.
        //
        setTimeout( function() {
          var height = (Math.max(
            document.body.scrollHeight, document.documentElement.scrollHeight,
            document.body.offsetHeight, document.documentElement.offsetHeight,
            document.body.clientHeight, document.documentElement.clientHeight
          ) + 'px');

          $('.modal-backdrop').closest('.modal').css({
            height: height
          });

          $('.modal-backdrop').css({
            position: 'absolute',
            top: 0,
            left: 0,
            width: '100%',
            height: height
          });
        }, 0);
      });
    }
  }]);
