angular.module('easyscott.clips', []).controller('ClipsController', [
    '$scope', '$filter', '$stateParams', 'WebsocketService', 'TimecodeService', 'DeviceModel', 'ClipModel', 'LiveClipModel', 'ClipMetadataTableCreator', 'ReportService', 'SystemPropertiesModel', 'HelperService',
    function($scope, $filter, $stateParams, WebsocketService, TimecodeService, DeviceModel, ClipModel, LiveClipModel, ClipMetadataTableCreator, ReportService, SystemPropertiesModel, HelperService) {

  $scope.devices = [];
  $scope.clips = [];
  $scope.totalClips = 0;
  $scope.filteredClips = 0;
  $scope.missedDeletes = [];
  $scope.missedChanges = [];

  $scope.currentExportFormat = null;
  $scope.currentExportFiletype = null;
  $scope.exportTargetVisible = false;
  $scope.exportTargetDeviceId = { id: 0 };
  $scope.exportTargetFilename = null;
  
  $scope.aleFile = null;
  $scope.aleDownloadFileName = null;
  $scope.aleMatchCount = 0;

  $scope.reportError = null;
  $scope.reportLoadingPercent = 0;
  $scope.reportLoadingFaker = null;
  
  $scope.soundReportHeader = {
    company: "",
    shootingDay: "",
    shootingDayDate: "",
    project: "",
    sets: "",
    comments: "",
    soundMixer: "",
    soundMixerContact: "",
    director: "",
    soundAssistant: "",
    boomOperators: "",
    productionManager: "",
    recorder: "",
    framerate: "",
    cameraFormat: "",
    bitDepth: "",
    samplingRate: "",
    referenceTone: "",
    fileType: "",
    memoryFolder: "",
  };

  $scope.dropdownOptions = {
    devices: [],
    rolls: [],
    shootingDays: [],
    storageDevices: [
      { id: 0, text: 'Download' },
      { id: -1, text: 'No USB storage available', type: 'USB storage', disabled: true }
    ]
  };
  
  $scope.dropdownSettings = {
    smartButtonMaxItems: 3,
    idProp: 'value',
    buttonClasses: "btn btn-info"
  };

  // Date picker elements
  $scope.$creationFrom = $("[name='creationFrom']");
  $scope.$creationTo = $("[name='creationTo']");

  $scope.refreshDeviceFilterDropdown = function() {
    var demand = $scope.getFilterDemand();
    demand.devices = '';
    jQuery.ajax({
      url: '/clipfilters/',
      data: demand
    })
    .success(function(data) {
      $scope.dropdownOptions.devices = _.chain(data.devices).sortBy('label').value();
    });
  };
  
  $scope.refreshRollFilterDropdown = function() {
    var demand = $scope.getFilterDemand();
    demand.rolls = '';
    jQuery.ajax({
      url: '/clipfilters/',
      data: demand
    })
    .success(function(data) {
      $scope.dropdownOptions.rolls = _.chain(data.rolls).sortBy('label').value();
    });
  };
  
  $scope.refreshProjectDayFilterDropdown = function() {
      var demand = $scope.getFilterDemand();
      demand.shootingDays = '';
      jQuery.ajax({
        url: '/clipfilters/',
        data: demand
      })
      .success(function(data) {
        $scope.dropdownOptions.shootingDays = _.chain(data.projectDays).sortBy('label').value();
      });
  };
  
  $scope.refreshCreationDateFilterDropdown = function() {
    var demand = $scope.getFilterDemand();
    demand.creationFrom = '';
    demand.creationTo = '';
    jQuery.ajax({
      url: '/clipfilters/',
      data: demand
    })
    .success(function(data) {
      $scope.$creationFrom.datepicker('setStartDate', new Date(data.minCreationTimestamp));
      $scope.$creationFrom.datepicker('setEndDate', new Date(data.maxCreationTimestamp));
      $scope.$creationTo.datepicker('setStartDate', new Date(data.minCreationTimestamp));
      $scope.$creationTo.datepicker('setEndDate', new Date(data.maxCreationTimestamp));
    });
  };
  
  $scope.dropdownEvents = {
    onItemSelect: function() { $scope.applyFilter(); },
    onItemDeselect: function() { $scope.applyFilter(); },
    onSelectAll: function() { $scope.applyFilter(); },

    // Previously selected items are not removed from the model at the time this event is fired.
    // Use setTimeout in order to let the handler finish and remove the items and call the applyFilter
    // function afterwards.
    onDeselectAll: function() { setTimeout(function() { $scope.applyFilter(); }, 1) }
  };

  $scope.loading = ClipModel.clipFetchInProgress;

  $scope.uploadAle = function() {
    if ($scope.aleFile == null) {
      return;
    }
    var data = new FormData();
    data.append('file', $scope.aleFile);

    $.ajax("/ale/", {
      type: 'POST',
      data: data,
      contentType: false,
      processData: false
    }).success(function(data) {
      $scope.aleMatchCount = data.matchedClipsCount;
      $scope.aleDownloadFileName = data.aleFileName;
    }).error(function() {
      $scope.aleMatchCount = 0;
      $scope.aleDownloadFileName = "na";
    });
  };

  $scope.generateAleDownloadLink = function() {
    return "/ale/?" + $.param({ fileName: $scope.aleDownloadFileName });
  };

  $scope.openMetadata = function(clip) {
    $('#clipMetadata').attr('data-clipId', clip.id);
    $('#clipMetadata').modal('show');
  };

  $scope.aleLoaded = function() {
    $('#aleForm input[type=file]').change(function() {
      $scope.aleFile = $('#aleForm input[type=file]').prop('files')[0];
    });
  };

  $scope.removeAllClips = function() {
    var demand = $scope.getFilterDemand();
    demand.v = ClipModel.currentViewId;
    
    var queryString = $.param(demand)

    // show busy indicator
    if($scope.filteredClips > 10) {
      $('#clipDeletionBusyIndicator').modal('show');
    }
    
    jQuery.ajax({
      method: 'DELETE',
      url: '/clips/?' + queryString,
    }).done(function() {
      // hide busy indicator
      $('#clipDeletionBusyIndicator').modal('hide');
      
      $scope.reset();
    });
  };

  $scope.removeClip = function(clipId) {
    var message = {
      request: 'CLIP_COMMAND',
      id: clipId,
      command: 'DELETE',
    };

    WebsocketService.sendMessage(message);
  };

  $scope.normalizeTC = function() {
    TimecodeService.normalizeTC($scope.filter.timecodeFrom);
    TimecodeService.normalizeTC($scope.filter.timecodeTo);
  }

  $scope.refreshState = function() {
    $scope.refreshDevices();
    $scope.refreshClips();
    $scope.refreshDropdownOptions();
  };

  $scope.refreshDevices = function() {
    $scope.devices = _.filter(DeviceModel.devices, function(device) {
      return device.type != 'websocket' && device.type != 'storage';
    });
    
    $scope.dropdownOptions.storageDevices = [];
    $scope.dropdownOptions.storageDevices = _.chain(DeviceModel.devices).filter(function(device) {
      return device.type == 'storage' && device.connected;
    }).map(function(device) {
      return { id: device.id, text: device.name, type: 'USB storage', disabled: false }
    }).value();
    if($scope.dropdownOptions.storageDevices.length == 0) {
      $scope.dropdownOptions.storageDevices.push({ id: -1, text: 'No USB storage available', type: 'USB storage', disabled: true });
    }
    $scope.dropdownOptions.storageDevices.unshift({ id: 0, text: 'Download' });
  };

  $scope.refreshClips = function() {
    // augment clips with source device name
    $scope.clips = _.map(ClipModel.clips, function(clip) {
      var sourceDevice = _.find($scope.devices, function(device) {
        return device.id == clip.sourceDeviceId;
      });

      if (sourceDevice) {
        clip.sourceDeviceName = sourceDevice.name;
      } else {
        clip.sourceDeviceName = clip.sourceDeviceId;
      }

      return clip;
    });
    $scope.totalClips = ClipModel.total;
    $scope.filteredClips = ClipModel.filtered;
  };

  $scope.refreshDropdownOptions = function() {
    // projectdays
    var demand = $scope.getFilterDemand();
    demand.shootingDays = '';
    jQuery.ajax({
      url: '/clipfilters/',
      data: demand
    })
    .success(function(data) {
      $scope.dropdownOptions.shootingDays = data.projectDays;
    });
  }

  $scope.applyFilter = function() {
    $scope.missedChanges = [];
    $scope.missedDeletes = [];
    
    var demand = $scope.getFilterDemand();
    ClipModel.setFilter(demand);
    
    LiveClipModel.setDemand(this.getFilterDemandObject());
    
    $scope.refreshDeviceFilterDropdown();
    $scope.refreshRollFilterDropdown();
    $scope.refreshProjectDayFilterDropdown();
    $scope.refreshCreationDateFilterDropdown();
  };

  $scope.reset = function() {
    $scope.$creationFrom.datepicker('update', '');
    $scope.$creationTo.datepicker('update', '');
    $scope.resetFilter();
    $scope.applyFilter();
  };

  $scope.requestTransferal = function() {
    var demand = $scope.getFilterDemand();
    jQuery.ajax({
      method: 'POST',
      url: '/clips/',
      data: demand
    });
  };

  $scope.resetFilter = function() {
    ClipModel.filter = null;

    $scope.filter = {
      devices: [],
      rolls: [],
      shootingDays: [],
      timecodeFrom: {
        hours: null,
        minutes: null,
        seconds: null,
        frames: null
      },
      timecodeTo: {
        hours: null,
        minutes: null,
        seconds: null,
        frames: null
      },
      creationFrom: null,
      creationTo: null
    };

    $scope.refreshState();
  };
  
  $scope.getFilterDemand = function() {
    var demand = {
      devices: '',
      rolls: '',
      shootingDays: '',
      timecodeFrom: '',
      timecodeTo: '',
    };

    // devices
    demand.devices = _.chain($scope.filter.devices)
      .map(function(device) {
        return device.id;
      })
      .join(',')
      .value();

    // rolls
    demand.rolls = _.chain($scope.filter.rolls)
      .map(function(camroll) {
        return camroll.id;
      })
      .join(',')
      .value();

    // shooting day
    demand.shootingDays = _.chain($scope.filter.shootingDays)
      .map(function(shootingDay) {
        return shootingDay.id
      })
      .join(',')
      .value();

    // timecode range
    demand.timecodeFrom = TimecodeService.render($scope.filter.timecodeFrom);
    demand.timecodeTo = TimecodeService.render($scope.filter.timecodeTo);

    // creation timestamp
    demand.creationFrom = $scope.filter.creationFrom;
    demand.creationTo = $scope.filter.creationTo;
    
    return demand;
  };
  
  $scope.getFilterDemandObject = function() {
    var demand = {
      devices: [],
      rolls: [],
      shootingDays: [],
      timecodeFrom: '',
      timecodeTo: '',
    };

    // devices
    demand.devices = _.chain($scope.filter.devices)
      .map(function(device) {
        return device.id;
      })
      .value();

    // rolls
    demand.rolls = _.chain($scope.filter.rolls)
      .map(function(camroll) {
        return camroll.id;
      })
      .value();

    // shooting day
    demand.shootingDays = _.chain($scope.filter.shootingDays)
      .map(function(shootingDay) {
        return shootingDay.id
      })
      .value();

    // timecode range
    demand.timecodeFrom = TimecodeService.render($scope.filter.timecodeFrom);
    demand.timecodeTo = TimecodeService.render($scope.filter.timecodeTo);

    // creation timestamp
    demand.creationFrom = $scope.filter.creationFrom;
    demand.creationTo = $scope.filter.creationTo;

    return demand;
  };
  
  $scope.prepareExport = function(exportFormat, fileType) {
    $scope.currentExportFormat = exportFormat;
    $scope.currentExportFiletype = fileType;
    $scope.exportTargetVisible = true;
  };
  
  $scope.runExport = function() {
    var fileName = _.trim($scope.exportTargetFilename).length > 0 ? $scope.exportTargetFilename : $scope.currentExportFormat + "-" + HelperService.isoDate();
    fileName += "." + $scope.currentExportFiletype;
    
    var demand = $scope.getFilterDemandObject();
    var data = { demand: demand };
    
    $scope.downloadReport($scope.currentExportFormat, data, fileName);
  };
  
  $scope.prepareSoundReport = function() {
    $scope.currentExportFiletype = 'xlsx';
    $scope.exportTargetVisible = false;
  };

  $scope.downloadSoundReport = function() {
      var fileName = _.trim($scope.exportTargetFilename).length > 0 ? $scope.exportTargetFilename : "Sound Report-" + HelperService.isoDate();
      fileName += "." + $scope.currentExportFiletype;
      var demand = $scope.getFilterDemandObject();
      var data = {
          demand: demand,
          headerData: $scope.soundReportHeader
      };

      $scope.downloadReport("soundReport", data, fileName);
  };

  $scope.downloadReport = function(reportType, reportData, downloadFileName) {
      if($scope.exportTargetDeviceId.id != 0) {
        reportData.targetDeviceId = $scope.exportTargetDeviceId.id;
      }
      reportData.filename = downloadFileName;

      $scope.startReportPercentFaker();
      ReportService.requestReport(reportType, reportData, function(error, fileName) {
          $scope.reportCreationFinished();
          $scope.$apply();

          if (error) {
              $scope.reportError = "Generating report failed";
              return;
          }

          if($scope.exportTargetDeviceId.id == 0) {
            ReportService.downloadReport(fileName, downloadFileName);
          }
      });
      $(window).scrollTo($(".clipExport"));
  };

  $scope.startReportPercentFaker = function() {
      $scope.reportError = null;
      $scope.reportLoadingPercent = 5;
      $scope.reportLoadingFaker = setInterval(function() {
          if ($scope.reportLoadingPercent > 0 && $scope.reportLoadingPercent < 95) {
              $scope.reportLoadingPercent += 1;
              $scope.$apply();
          }
      }, 500);
  };

  $scope.stopReportPercentFaker = function() {
      if ($scope.reportLoadingFaker) {
          clearTimeout($scope.reportLoadingFaker);
      }
  };

  $scope.reportCreationFinished = function() {
      $scope.exportTargetVisible = false;
      $scope.stopReportPercentFaker();
      $scope.reportLoadingPercent = 100;
      setTimeout(function() {
          $scope.reportLoadingPercent = 0;
          $scope.$apply();
      }, 1000);
  };
  
  $scope.lazyLoad = function() {
    $scope.loading = true;
    ClipModel.loadNextPage();
  };
  
  $scope.requestSystemProperties = function() {
    var message = {
      request: 'REQUEST_SYSTEM_PROPERTIES'
    };
    WebsocketService.sendMessage(message);
  };

  DeviceModel.addEventListener('change', function(type) {
    $scope.refreshState();
    $scope.$apply();
  });

  ClipModel.addEventListener('change', function(type) {
    $scope.refreshState();
    $scope.$apply();
  });

  ClipModel.addEventListener('clipFetchStarted', function() {
    $scope.loading = true;
  });

  ClipModel.addEventListener('clipFetchFinished', function() {
    $scope.loading = false;
    $scope.refreshState();
  });
  ClipModel.addEventListener('clipRemoved', function(type, clipId) {
    $scope.missedDeletes.push(clipId);
    $scope.$apply();
  });
  
  LiveClipModel.addEventListener('clipUpdate', function(type, changedClipIds) {
    $scope.missedChanges = _.uniq($scope.missedChanges.concat(changedClipIds));
    $scope.$apply();
  });
  SystemPropertiesModel.addEventListener('change', function(type) {
    if (SystemPropertiesModel.properties.soundReportHeader) {
      $scope.soundReportHeader = JSON.parse(SystemPropertiesModel.properties.soundReportHeader);
      $scope.$apply();
    }
  });

  $scope.resetFilter();
  
  if($stateParams.device) {
    $scope.filter.devices = [ {id: $stateParams.device} ];
  }
  
  $scope.requestSystemProperties();
  $scope.refreshState();
  $scope.applyFilter();

  // Configure date pickers
  var datePickerOptions = {
      clearBtn: true,
      todayBtn: "linked",
      autoclose: true,
      format: 'yyyy-mm-dd'
  };
  $scope.$creationFrom.datepicker(datePickerOptions);
  $scope.$creationTo.datepicker(datePickerOptions);

  $scope.$creationFrom.on("changeDate", function(e) {
          $scope.filter.creationFrom = e.date ? e.date.valueOf() : null;
          $scope.applyFilter();
      });

  $scope.$creationTo.on("changeDate", function(e) {
          $scope.filter.creationTo = e.date ? e.date.valueOf() + 24*60*60*1000 : null;
          $scope.applyFilter();
  });

  // Configure date picker in sound report modal
  $('#soundReport').on('show.bs.modal', function() {
    // Use common date picker options but also allow
    // arbitrary user inputs.
    var options = $.extend({}, datePickerOptions);
    options.forceParse = false;

    $("#txtShootingDayDate").datepicker(options);
  });

  // Metadata Modal Dialog
  $scope.$on('$viewContentLoaded', function() {
    $('#clipMetadata').on('show.bs.modal', function() {
      var $element = $(this);
      var $body = $element.find('.modal-body');
      var $table = $body.find('table');

      var clipId = $element.attr('data-clipId');
      var clip = _.filter($scope.clips, function(clip) {
        return clip.id == clipId;
      })[0];

      // Clean dialog
      $table.empty();
      $body.find('i').remove();
      $body.find('p').remove();

      // Show spinner
      $body.append($('<i>').addClass('fa fa-spinner fa-spin'));

      if (clip != null) {
        ClipMetadataTableCreator.createTable(clip, function(table) {
          $table.replaceWith(table);

          // Remove spinners
          $body.find('.fa-spinner').remove();
        });
      } else {
        $body.find('.fa-spinner').replaceWith($('<p>').text('Sorry, cannot get the clip metadata'));
      }
    });
  });
  
  $scope.createSoundReport = function() {
    $scope.downloadSoundReport();
    jQuery('#soundReport').modal('hide');
  };

}]);
