angular.module('easyscott.devices', [])
  .controller('DevicesController', [ '$scope', '$q', 'WebsocketService', 'DeviceModel', 'ChannelModel', 'HelperService', 'CjamSettingsService', 'SystemPropertiesModel', 'TimecodeService', function($scope, $q, WebsocketService, DeviceModel, ChannelModel, HelperService, CjamSettingsService, SystemPropertiesModel, TimecodeService) {
    $scope.dynamicChannelTypes = ['alexa', 'arricap2', 'red', 'cube', 'vitec', 'clink', 'zeiss'];

    $scope.channels = [];
    $scope.devices = [];
    $scope.dynamicChannels = [];
    $scope.directDevices = {};
    $scope.dummyDevices = {};
    $scope.cJamConnectedDevices = [];
    $scope.childDevices = {};
    $scope.orphanDevices = [];
    $scope.isBusy = false;
    $scope.acnChannel = "?";
    $scope.localDevice = null;
    $scope.advancedFeaturesLocked = true;
    $scope.unitId = "";
    $scope.showDetails = localStorage.getItem('showDetails') === 'true';
    $scope.editables = {};

    // For easy layer-breaking access from the template
    $scope.cjam = CjamSettingsService;

    $scope.status = {
      framerate: {},
      projectFramerate: {},
      channel: {},
      cjamRunning: false,
      timecodeSet: false,
      userbits: {
        one: '00',
        two: '00',
        three: '00',
        four: '00'
      },
      tunePwm: ''
    };

    $scope.refreshInterval = null;

    $scope.refreshDynamicChannels = function() {
      $scope.dynamicChannels = $scope.channels.filter(function(channel) {
        return $.inArray(channel.type, $scope.dynamicChannelTypes) >= 0;
      });
    };
    $scope.refreshDirectDevices = function() {
      $scope.directDevices = {};
      $scope.channels.forEach(function(channel) {
        $scope.directDevices[channel.type] = $scope.devices.filter(function(device) {
          return device.type == channel.type && !device.parentDeviceId;
        });
      });
    };
    $scope.refreshCJamConnectedDevices = function() {
      // Add child devices
      var childDevices = $scope.devices
          .filter(function(device) {
            return device.parentDeviceId != null;
          })
          .map(function(childDevice) {
            var parentDevice = $scope.devices.filter(function(device) {
              return device.id == childDevice.parentDeviceId;
            })[0];
            return {
              parent: parentDevice,
              child: childDevice
            };
          }).filter(function(devicePair) {
            return devicePair.child != null && devicePair.parent != null;
          });

      $scope.childDevices = {};
      childDevices.forEach(function(devicePair) {
          if (!$scope.childDevices[devicePair.parent.id]) {
              $scope.childDevices[devicePair.parent.id] = [];
          }
          $scope.childDevices[devicePair.parent.id].push(devicePair.child);
      });

      // Add Lockits with a LANC camera connected
      var lockitsWithLanc = $scope.devices
          .filter(function(device) {
            return device.type == "acn"
              && device.connected
              && device.deviceTypes
              && device.deviceTypes.indexOf("CAMERA") >= 0;
          })
          .map(function(lockit) {
            return {
              parent: lockit,
              child: {
                id: "LANC",
                name: "LANC Camera",
                type: "lanc",
                connected: true
              }
            };
          });

      var cJamConnectedDevices = childDevices.concat(lockitsWithLanc);
      $scope.cJamConnectedDevices = HelperService.mergeArrays($scope.cJamConnectedDevices, cJamConnectedDevices, function(a, b) {
        return a.parent.id === b.parent.id && a.child.id === b.child.id;
      });
    };

    $scope.refreshOrphanDevices = function() {
        $scope.orphanDevices = $scope.devices.filter(function(device) {
            if (device.type === 'Timecode') {
                // The internal Lockit
                return false;
            }

            if ($scope.generator && device.parentDeviceId) {
              return true;
            }

            var channel = $scope.channels.filter(function(channel) {
                return channel.type === device.type;
            })[0];
            var parentDevice = $scope.devices.filter(function(parent) {
                return parent.id === device.parentDeviceId;
            })[0];
            return (!device.parentDeviceId && !channel) || (device.parentDeviceId && !parentDevice);
        });
    };

    $scope.dummyDeviceForChannel = function(channel) {
      // Cache dummy devices for each channel type in order to
      // prevent creating too much new objects and to stop
      // Angular.js from complaining about it.
      if (!$scope.dummyDevices[channel.type]) {
        $scope.dummyDevices[channel.type] = {
          connected: false,
          energyLevel: "UNKNOWN",
          id: "00:00:00:00:00:00",
          name: channel.name,
          properties: {},
          removed: false,
          type: channel.type,
          request: "DEVICE_STATE"
        };
      }
      return $scope.dummyDevices[channel.type];
    };
    $scope.updateDevice = function(device) {
      var message = {
        request: 'RENAME_DEVICE',
        id: device.id,
        name: device.name
      };

      WebsocketService.sendMessage(message);
    };

    $scope.removeDevice = function(deviceId) {
      var message = {
        request: 'DEVICE_COMMAND',
        id: deviceId,
        command: 'DELETE'
      };

      $scope.doBusy(function(done) {
        $scope.executeSynchronousRequest(message, done);
      });

      var children = $scope.childDevices[deviceId] || [];
      children.forEach(function(child) {
          $scope.removeDevice(child.id);
      });
    };

    $scope.removeAllLockitDevices = function() {
        var lockits = $scope.devices.filter(function(d) { return d.type === 'acn' });
        lockits.forEach(function(d) {
            $scope.removeDevice(d.id);
        });
        $scope.advancedFeaturesLocked = true;
    };

    $scope.removeChannel = function(channelId) {
      var channel = $scope.channels.filter(function(channel) {
        return channel.id == channelId;
      })[0];

      var message = {
        request: 'CHANNEL_COMMAND',
        id: channelId,
        command: 'DELETE'
      };

      var d = $q.defer();

      $scope.doBusy(function(done) {
        var callback = function() {
          done();
          d.resolve();
        };
        $scope.executeSynchronousRequest(message, callback);
      });

      return d.promise;
    };

    var removeOldActiveChannel = function(callback) {
      var oldChannels = $scope.channels.filter(function(channel) {
        return channel.type == 'alexa' || channel.type == 'arricap2' || channel.type == 'clink' || channel.type == 'red';
      });

      var promises = [];

      for(var i = 0; i < oldChannels.length; i++) {
        promises.push($scope.removeChannel(oldChannels[i].id));
      }

      $q.all(promises).then(callback);
    };

    // CHANNEL FUNCTIONS

    $scope.addAlexaChannel = function() {
      if (!$scope.newAlexa.name) {
        $scope.newAlexa.error = "Please choose a name for your ARRI Alexa.";
        return;
      }
      if (!$scope.newAlexa.serial) {
        $scope.newAlexa.error = "Please enter the serial number of your ARRI Alexa.";
        return;
      }

      $scope.newAlexa.error = null;
      jQuery('#dlgAlexa').modal('hide');

      removeOldActiveChannel(function() {
        var alexaSerial = $scope.normalizeAlexaSerial($scope.newAlexa.serial);
        var message = {
          request: 'CREATE_CHANNEL',
          type: 'alexa',
          name: $scope.newAlexa.name,
          configuration: {
            hostname: 'alexa' + alexaSerial + '.local'
          }
        };

        $scope.doBusy(function(done) {
          $scope.executeSynchronousRequest(message, done);
        });
      });
    };

    $scope.addAmiraChannel = function() {
      if (!$scope.newAmira.name) {
        $scope.newAmira.error = "Please choose a name for your ARRI CAP2 device.";
        return;
      }
      if (!$scope.newAmira.password) {
        $scope.newAmira.error = "Please enter the password of your ARRI CAP2 device.";
        return;
      }

      $scope.newAmira.error = null;
      jQuery('#dlgAmira').modal('hide');

      removeOldActiveChannel(function() {
        var message = {
          request: 'CREATE_CHANNEL',
          type: 'arricap2',
          name: $scope.newAmira.name,
          configuration: {
            password: $scope.newAmira.password
          }
        };

        $scope.doBusy(function(done) {
          $scope.executeSynchronousRequest(message, done);
        });
      });
    };

    $scope.addRedChannel = function() {
      if ($scope.newRed.name.length == 0) {
        return;
      }

      jQuery('#dlgRed').modal('hide');

      removeOldActiveChannel(function() {
        var message = {
          request: 'CREATE_CHANNEL',
          type: 'red',
          name: $scope.newRed.name
        };

        $scope.doBusy(function(done) {
          $scope.executeSynchronousRequest(message, done);
        });
      });
    };

    $scope.addCLinkChannel = function() {
      if ($scope.newCLink.name.length == 0) {
        return;
      }

      jQuery('#dlgCLink').modal('hide');

      removeOldActiveChannel(function() {
        var message = {
          request: 'CREATE_CHANNEL',
          type: 'clink',
          name: $scope.newCLink.name,
          configuration: {}
        };

        $scope.doBusy(function(done) {
          $scope.executeSynchronousRequest(message, done);
        });
      });
    };

    jQuery('#dlgCube').modal('hide');

    $scope.addCubeChannel = function() {
      if ($scope.newCube.hostname.length == 0 || $scope.newCube.name.length == 0) {
        return;
      }

      removeOldActiveChannel(function() {
        var message = {
          request: 'CREATE_CHANNEL',
          type: 'cube',
          name: $scope.newCube.name,
          configuration: {
            hostname: $scope.newCube.hostname
          }
        };

        $scope.doBusy(function(done) {
          $scope.executeSynchronousRequest(message, done);
        });
      });
    };

    $scope.addVitecChannel = function() {
      if ($scope.newVitec.hostname.length == 0 || $scope.newVitec.name.length == 0) {
        return;
      }

      jQuery('#dlgVitec').modal('hide');

      removeOldActiveChannel(function() {
        var message = {
          request: 'CREATE_CHANNEL',
          type: 'vitec',
          name: $scope.newVitec.name,
          configuration: {
            hostname: $scope.newVitec.hostname
          }
        };

        $scope.doBusy(function(done) {
          $scope.executeSynchronousRequest(message, done);
        });
      });
    };

    $scope.resetDialogs = function() {
      $scope.newACN = { name: 'ACN/LockIt', mode: '' };
      $scope.newAlexa = { name: 'Alexa', serial: '', error: null };
      $scope.newAmira = { name: 'ARRI CAP2', password: 'arri', rollPrefix: 'A', error: null},
      $scope.newRed = { name: 'RED DSMC' };
      $scope.newCube = { name: 'Cube Recorder', hostname: ''};
      $scope.newVitec = { name: 'Vitec Recorder', hostname: ''};
      $scope.newCLink = { name: 'Sound Devices' };
    };

    $scope.refreshState = function() {
      $scope.devices = _.sortBy(DeviceModel.devices, function(device) {
        return device.name + device.id;
      });
      $scope.channels = ChannelModel.channels;

      $scope.refreshCJamConnectedDevices();
      $scope.refreshDynamicChannels();
      $scope.refreshDirectDevices();
      $scope.refreshOrphanDevices();
      $scope.dummyDevices = {};

      var masterClock = DeviceModel.getMasterclockDevice();
      $scope.localDevice = masterClock;
      if (masterClock) {
        $scope.acnChannel = masterClock.properties.acnChannel;
        $scope.unitId = masterClock.properties.unitId;
        if ($scope.unitId && !$scope.editables.unitId) {
            $scope.editables.unitId = $scope.unitId;
        }
      }
    };

    $scope.broadcastAcnTune = function() {
      if (confirm("Attention!\n\nTune will interrupt operation of all slaves for several minutes!\nDo not activate if pending or active recording!\n\nContinue?")) {
        WebsocketService.sendMessage({
          request: 'BROADCAST_ACN_TUNE_MESSAGE'
        });
      }
      $scope.advancedFeaturesLocked = true;
    };

    $scope.broadcastAcnReferenceTune = function() {
      if (confirm("Attention!\n\nCalibration will permanently reset the default reference.\n\nOnly activate with known good reference on Master.\n\nThis process may take up to 15 Minutes and interrupt output signals. Do NOT switch off Master and slaves until they start blinking green again.\n\nContinue?")) {
        WebsocketService.sendMessage({
          request: 'BROADCAST_ACN_REFERENCE_TUNE_MESSAGE'
        });
      }
      $scope.advancedFeaturesLocked = true;
    };

    $scope.resetCjam = function() {
      WebsocketService.sendMessage({
        request: 'CHANNEL_COMMAND',
        id: ChannelModel.getActiveACNChannelId(),
        command: 'RESET_CJAM'
      });
    };

    $scope.toggleAdvancedFeatureLock = function() {
        $scope.advancedFeaturesLocked = !$scope.advancedFeaturesLocked;
    };

    $scope.referenceTuneAllowed = function() {
      return (SystemPropertiesModel.properties || {})['fkReferenceTune'] === 'true';
    };

    $scope.resetCjamAllowed = function() {
      return (SystemPropertiesModel.properties || {})['fkResetCjam'] === 'true';
    };

    $scope.editUnitId = function() {
        $scope.editables.unitId = $scope.unitId;
        $('#dlgUnitId').modal('show');
    };

    $scope.saveUnitId = function() {
        var message = {
            request: 'CHANNEL_COMMAND',
            id: ChannelModel.getActiveACNChannelId(),
            command: 'SET_UNIT_ID',
            arguments: {
                unitId: $scope.editables.unitId
            }
        };
        WebsocketService.sendMessage(message);
    };

    $scope.setShowDetails = function(v) {
        $scope.showDetails = v;
        localStorage.setItem('showDetails', v ? 'true' : 'false');
    };

    $scope.updateChannel = function() {
      CjamSettingsService.setChannel($scope.editables.channel.value);
      $scope.status.channel = $scope.editables.channel;
      $scope.editables.channel = null;
    };

    $scope.updateTunePwm = function() {
      var channelId = ChannelModel.getActiveACNChannelId();
      if (channelId) {
        var message = {
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_TUNE_PWM',
          arguments: {
            tunepwm: $scope.editables.tunePwm
          }
        }
      }
      WebsocketService.sendMessage(message);

      $scope.status.tunePwm = $scope.editables.tunePwm;
      $scope.editables.tunePwm = false;
    };

    $scope.lastSeenDiff = function(device) {
      var refTcS = device.properties['lastSeenTimecode'];
      if (!refTcS) { return null; }
      var refTc = TimecodeService.parse(refTcS);
      if (!refTc) { return null; }
      var nowTc = $scope.currentTc();
      if (!nowTc) { return null; }
      return TimecodeService.difference(nowTc, refTc);
    };

    $scope.lastSeenNanoDiff = function(device) {
      var nanos = JSON.parse(device.properties['lastSeenNanos']);
      if (!nanos || nanos.length === 0) { return null; }
      var nano = nanos[nanos.length - 1];
      return $scope.nanoDiff(nano) / 1000000;
    };

    $scope.syncIntervalAvg = function(device) {
      var nanos = JSON.parse(device.properties['lastSeenNanos']);
      if (!nanos || nanos.length === 0) { return null; }

      var sum = 0;
      var count = 0;
      for (var i = 1; i < nanos.length; i++) {
        if ($scope.nanoDiff(nanos[i]) / 1000000 < 60000) {
          sum += (nanos[i] - nanos[i - 1]);
          count += 1;
        }
      }
      var currentDiff = $scope.nanoDiff(nanos[nanos.length - 1]);
      if (currentDiff / 1000000 > 6000) {
        sum += currentDiff;
        count += 1;
      }
      return (sum / count) / 1000000;
    };

    $scope.lastSeenPartialName = function(device) {
      var diff = $scope.lastSeenDiff(device);
      if (diff <= 7) {
        return "signal5";
      } else if (diff <= 14) {
        return "signal4";
      } else if (diff <= 21) {
        return "signal3";
      } else if (diff <= 28) {
        return "signal2";
      } else {
        return "signal1"
      }
    };

    // HELPERS

    $scope.normalizeAlexaSerial = function(serial) {
      while (serial.toString().length < 4) {
        serial = "0" + serial;
      }
      return serial;
    };

    $scope.doBusy = function(f) {
        var done = function() {
            $scope.isBusy = false;
            $scope.$apply();
        };

        $scope.isBusy = true;
        f(done);
    };

    DeviceModel.addEventListener('change', function(type) {
      CjamSettingsService.updateCjamStatus($scope.status);
      $scope.refreshState();
      $scope.$apply();
    });

    ChannelModel.addEventListener('change', function(type) {
      $scope.refreshState();
      $scope.$apply();
    });

    CjamSettingsService.updateCjamStatus($scope.status);

    SystemPropertiesModel.addEventListener('change', function(type) {
      $scope.$apply();
    });


    $scope.$on('$destroy', function () {
      if ($scope.refreshInterval) {
        clearInterval($scope.refreshInterval);
        $scope.refreshInterval = null;
      }
    });

    // $scope.refreshInterval = setInterval(function() {
    //   $scope.$apply();
    // }, 1000);

    $scope.resetDialogs();
    $scope.refreshState();
  }]);
