angular.module('easyscott.fakeclink', [])
  .controller('FakeCLinkController', [ '$rootScope', '$scope', 'WebsocketService', 'ChannelModel', 'DeviceModel', 'ClipModel', 'SystemPropertiesModel', 'SlateModel', function($rootScope, $scope, WebsocketService, channelModel, deviceModel, ClipModel, SystemPropertiesModel, SlateModel) {
    $scope.recording = false;
    $scope.loadingRecord = false;
    $scope.loadingTracks = false;
    $scope.loadingMeters = true;
    $scope.loadingDeviceData = false;
    $scope.loadingClips = false;
    $scope.loadingImport = false;
    $scope.loadingBulk = false;
    $scope.loadingTake = false;
    $scope.bulkLoadingFinished = false;
    $scope.showMeters = false;
    $scope.showTracks = false;
    $scope.showBulkImport = false;
    $scope.showHistory = false;
    $scope.clips = [];
    $scope.meters = [];
    $scope.batchSize = 100;
    $scope.clip = {
      current: {
        filename: null,
        takeId: null,
        scenelabel: null,
        takelabel: null,
        takeNumber: null,
        comment: null,
        tracks: []
      },
      next: {
        filename: null,
        takeId: null,
        scenelabel: null,
        takelabel: null,
        takeNumber: null,
        comment: null
      }
    };
    $scope.editTakeModel = {};
    $scope.tracks = [];
    $scope.clinkDevice = {
        connected: true,
        properties: {
            hasRequiredFirmware: 'true',
            currentFilename: '1-3T02',
            currentTakeIdentifier: '1',
            currentSceneLabel: '1-3',
            currentTakeLabel: '2',
            currentNotes: '',
            currentCircled: true,
            nextFilename: '1-3T03',
            nextTakeIdentifier: '2',
            nextSceneLabel: '1-3',
            nextTakeLabel: '3',
            nextNotes: '',
            recording: false,
            currentTracks: JSON.stringify([1]),
            trackTitles: JSON.stringify(["L", "R", "A", "B", "C", "D", "E", "F", "G", "H", "X1", "X2"]),
            trackNames: JSON.stringify(["L", "R", "A", "B", "C", "D", "E", "F", "G", "H", "X1", "X2"]),
            armingBitmap: 0xffff
        },
        state: 'OK'
    };
    $scope.hasSameTcClips = false;
    $scope.sysProperties = SystemPropertiesModel.properties;
    $scope.editables = {
        shouldApplySlateInfo: $scope.sysProperties.applySlateInfoToSoundRecorder === 'true'
    };

    SystemPropertiesModel.addEventListener('change', function(type) {
        $scope.sysProperties = SystemPropertiesModel.properties;
        $scope.editables.shouldApplySlateInfo = $scope.sysProperties.applySlateInfoToSoundRecorder === 'true';
        $scope.$apply();
    });

    setTimeout(function() {
        SlateModel.addEventListener('change', function (type) {
            if ($scope.editables.shouldApplySlateInfo) {
                console.log(SlateModel.upcomingSlateInfo);
                if (SlateModel.upcomingSlateInfo.slatingType === 'us') {
                  if (SlateModel.upcomingSlateInfo.slate) {
                    $scope.clinkDevice.properties.nextSceneLabel = SlateModel.upcomingSlateInfo.slate;
                  }
                } else {
                  if (SlateModel.upcomingSlateInfo.scene && SlateModel.upcomingSlateInfo.slate) {
                    $scope.clinkDevice.properties.nextSceneLabel =
                        [SlateModel.upcomingSlateInfo.scene, SlateModel.upcomingSlateInfo.slate].join("-")
                  }
                }
                if (SlateModel.upcomingSlateInfo.take) {
                    $scope.clinkDevice.properties.nextTakeLabel = SlateModel.upcomingSlateInfo.take;
                }
                $scope.clinkDevice.properties.nextFilename = $scope.clinkDevice.properties.nextSceneLabel + 'T' + leftPad($scope.clinkDevice.properties.nextTakeLabel + '', 2, '0');
                updateClipSettings();
                $scope.$apply();
            }
        })
    }, 2000);

    $scope.isReady = function() {
      return $scope.clinkDevice && $scope.clip.next.takelabel;
    };

    $scope.toggleTracks = function() {
      if ($scope.showTracks) {
        $scope.showTracks = false;
      } else {
        $scope.showTracks = true;
      }
    };

    $scope.tracksBusyIndicator = {
      show: function() {
        $scope.loadingTracks = true;
      },
      hide: function() {
        $scope.loadingTracks = false;
      }
    };

    $scope.deviceDataBusyIndicator = {
        show: function() {
          $scope.loadingDeviceData = true;
        },
        hide: function() {
          $scope.loadingDeviceData = false;
        }
    };

    $scope.clipsBusyIndicator = {
        show: function() {
          $scope.loadingClips = true;
        },
        hide: function() {
          $scope.loadingClips = false;
        }
    };

    $scope.recordBusyIndicator = {
        show: function() {
          $scope.loadingRecord = true;
        },
        hide: function() {
          $scope.loadingRecord = false;
        }
    };

    $scope.editDialogBusyIndicator = {
        show: function() {
          $scope.loadingTake = true;
        },
        hide: function() {
          $scope.loadingTake = false;
        }
    };

    $scope.refreshClips = function() {
    };

    $scope.importClips = function(clipData) {
    };

    var updateClipSettings = function() {
      if($scope.clinkDevice) {
        localStorage.setItem('fakeclink', JSON.stringify($scope.clinkDevice));

        $scope.clinkDevice.remoteControllable = $scope.clinkDevice.connected && $scope.clinkDevice.properties.hasRequiredFirmware == 'true';
        $scope.clinkDevice.clipsWithZeroTimecode = $scope.clinkDevice.connected && $scope.clinkDevice.properties.clipsWithZeroTimecode == 'true';

        // current clip
        $scope.clip.current.filename = $scope.clinkDevice.properties.currentFilename;
        $scope.clip.current.takeId = $scope.clinkDevice.properties.currentTakeIdentifier;
        $scope.clip.current.scenelabel = $scope.clinkDevice.properties.currentSceneLabel;
        $scope.clip.current.takelabel = $scope.clinkDevice.properties.currentTakeLabel;
        $scope.clip.current.takeNumber = parseInt($scope.clip.current.takelabel);
        $scope.clip.current.comment = $scope.clinkDevice.properties.currentNotes;
        $scope.clip.current.circle = JSON.parse($scope.clinkDevice.properties.currentCircled);
        $scope.clip.current.tracks = JSON.parse($scope.clinkDevice.properties.currentTracks);

        // next clip
        $scope.clip.next.filename = $scope.clinkDevice.properties.nextFilename;
        $scope.clip.next.takeId = $scope.clinkDevice.properties.nextTakeIdentifier;
        $scope.clip.next.scenelabel = $scope.clinkDevice.properties.nextSceneLabel;
        $scope.clip.next.takelabel = $scope.clinkDevice.properties.nextTakeLabel;
        $scope.clip.next.takeNumber = parseInt($scope.clip.next.takelabel);
        $scope.clip.next.comment = $scope.clinkDevice.properties.nextNotes;

        // recording
        $scope.recording = JSON.parse($scope.clinkDevice.properties.recording);

        // tracks
        var tracks = [];
        var trackTitles = JSON.parse($scope.clinkDevice.properties.trackTitles);
        var trackNames = JSON.parse($scope.clinkDevice.properties.trackNames);

        for (var i = 0; i < trackNames.length; i++) {
          tracks.push({
            index: i,
            title: trackTitles[i],
            name: trackNames[i]
          });
        }
        $scope.tracks = tracks;
      }
    };

    $scope.editTake = function(clip, busyIndicator) {
    };

    $scope.updateScene = function(clip, busyIndicator) {
    };

    $scope.updateTake = function(clip, busyIndicator) {
        if (clip.takeId === '1') {
            $scope.clinkDevice.properties.currentSceneLabel = clip.scenelabel;
            $scope.clinkDevice.properties.currentTakeLabel = clip.takeNumber;
            $scope.clinkDevice.properties.currentFilename = clip.scenelabel + 'T' + leftPad(clip.takeNumber + '', 2, '0');
            updateClipSettings();
        } else if (clip.takeId === '2') {
            $scope.clinkDevice.properties.nextSceneLabel = clip.scenelabel;
            $scope.clinkDevice.properties.nextTakeLabel = clip.takeNumber;
            $scope.clinkDevice.properties.nextFilename = clip.scenelabel + 'T' + leftPad(clip.takeNumber + '', 2, '0');
            updateClipSettings();
        }
    };

    $scope.updateNotes = function(clip, busyIndicator) {
        if (clip.takeId === '1') {
            $scope.clinkDevice.properties.currentNotes = clip.comment;
            updateClipSettings();
        } else if (clip.takeId === '2') {
            $scope.clinkDevice.properties.nextNotes = clip.comment;
            updateClipSettings();
        }
    };

    $scope.updateCircle = function(clip, busyIndicator) {
        if (clip.takeId === '1') {
            $scope.clinkDevice.properties.currentCircled = clip.circle;
            updateClipSettings();
        } else if (clip.takeId === '2') {
            $scope.clinkDevice.properties.nextCircled = clip.circle;
            updateClipSettings();
        }
    };

    $scope.updateClipTrackName = function(clip, track, busyIndicator) {
    };

    $scope.updateTrackName = function(track, busyIndicator) {
        var names = JSON.parse($scope.clinkDevice.properties.trackNames);
        names[track.index] = track.name;
        $scope.clinkDevice.properties.trackNames = JSON.stringify(names);
        updateClipSettings();
    };

    $scope.falseTake = function(clip) {
    };

    $scope.startRecording = function() {
    };

    $scope.stopRecording = function() {
    };

    $scope.toggleHistory = function() {
      if($scope.showHistory) {
        $scope.disableHistory();
      } else {
        $scope.enableHistory();
      }
    };
    $scope.enableHistory = function() {
      $scope.showHistory = true;
      $scope.refreshClips();
    };
    $scope.disableHistory = function() {
      $scope.showHistory = false;
    };

    $scope.toggleBulkImport = function() {
      if($scope.showBulkImport) {
        $scope.disableBulkImport();
      } else {
        $scope.enableBulkImport();
      }
    };
    $scope.enableBulkImport = function() {
      $scope.showBulkImport = true;
    };
    $scope.disableBulkImport = function() {
      $scope.showBulkImport = false;
    };

    $scope.isTrackArmed = function(index) {
      if ($scope.clinkDevice) {
        var armingBitmap = parseInt($scope.clinkDevice.properties.armingBitmap);
        return ((armingBitmap >> index) & 1) != 0;
      }

      return false;
    };

    $scope.bulkImport = function() {
    };

    $scope.fullRefresh = function() {
    };

    $scope.switchArmState = function(trackIndex) {
        var bitmap = $scope.clinkDevice.properties.armingBitmap;
        if ($scope.isTrackArmed(trackIndex)) {
            bitmap &= ~(1 << trackIndex);
        } else {
            bitmap |= (1 << trackIndex);
        }
        $scope.clinkDevice.properties.armingBitmap = bitmap;
    };

    $scope.updateSlateInfoAcceptance = function(event) {
    };

    var fakeDevice = localStorage.getItem('fakeclink');
    if (fakeDevice) {
        $scope.clinkDevice = JSON.parse(fakeDevice);
    }
    updateClipSettings();

    var leftPad = function(s, n, c) {
        while (s.length < n) {
            s = c + s;
        }
        return s;
    };

    /*
     ============================ FIX FOR MOBILE SAFARI ==============================
     Modified version of the workaround described in this blog post:
     http://www.abeautifulsite.net/bootstrap-3-modals-and-the-ios-virtual-keyboard/

     This works around a bug in mobile Safari described here:
     https://github.com/twbs/bootstrap/issues/9023
     */

    // iOS check...ugly but necessary
    if( navigator.userAgent.match(/iPhone|iPad|iPod/i) ) {
      $('.modal').on('show.bs.modal', function() {
        // Position modal absolute and bump it down to the scrollPosition
        $(this)
          .css({
            position: 'absolute',
            marginTop: $(window).scrollTop() + 'px',
            bottom: 'auto'
          });

        // Position backdrop absolute and make it span the entire page
        //
        // Also dirty, but we need to tap into the backdrop after Boostrap 
        // positions it but before transitions finish.
        //
        setTimeout( function() {
          var height = (Math.max(
            document.body.scrollHeight, document.documentElement.scrollHeight,
            document.body.offsetHeight, document.documentElement.offsetHeight,
            document.body.clientHeight, document.documentElement.clientHeight
          ) + 'px');

          $('.modal-backdrop').closest('.modal').css({
            height: height
          });

          $('.modal-backdrop').css({
            position: 'absolute',
            top: 0,
            left: 0,
            width: '100%',
            height: height
          });
        }, 0);
      });
    }
  }]);
