angular.module('easyscott.lens', []).controller('LensController', [ '$scope', 'WebsocketService', 'ChannelModel', 'TimecodeService', 'DeviceModel', function($scope, WebsocketService, channelModel, TimecodeService, DeviceModel) {

  var updateLiveExportUIFromChannelConfiguration = function() {
    var zeissChannel = channelModel.getChannelById(channelModel.getActiveZeissChannelId());
    if(zeissChannel) {
      var config = zeissChannel.configuration;

      $scope.liveexport.running = config.liveExport === 'true';

      if (config.liveExportFilename) {
        $scope.liveexport.targetFilename = config.liveExportFilename.substring(0, config.liveExportFilename.length - 4);
      } else {
        $scope.liveexport.targetFilename = null;
      }

      if (config.liveExportStorageDeviceID) {
        var formattedDeviceId = config.liveExportStorageDeviceID.match(/.{2}/g).join(':').trim();
        $scope.liveexport.targetDeviceId = _.find($scope.dropdownOptions.storageDevices, function (d) {
          return d.id == formattedDeviceId;
        });
      } else {
        $scope.liveexport.targetDeviceId = null;
      }
    }
  };

  var startOfDay = new Date();
  startOfDay.setHours(0, 0, 0, 0);

  $scope.busy = false;

  $scope.filter = {
    devices: [],
    timecodeFrom: {
      hours: "0"
    },
    timecodeTo: {
      hours: "23",
      minutes: "59",
      seconds: "59",
      frames: "29"
    },
    creationFrom: startOfDay.valueOf(),
    creationTo: startOfDay.valueOf()
  };

  $scope.dropdownOptions = {
    devices: [],
    storageDevices: [
      { id: 0, text: 'Download' },
      { id: -1, text: 'No USB storage available', type: 'USB storage', disabled: true }
    ],
    usbDevices: [
      { id: -1, text: 'No USB storage available', type: 'USB storage', disabled: true }
    ]
  };

  $scope.exportTargetDeviceId = { id: 0 };


  $scope.liveexport = {
    targetDeviceId: { id: -1 },
    targetFilename: '',
    running: false,
    setRunning: function(state) {
      WebsocketService.sendMessage({
        request: 'CHANNEL_COMMAND',
        id: channelModel.getActiveZeissChannelId(),
        command: 'LIVE_EXPORT',
        arguments: {
          enabled: state,
          storageDevice: $scope.liveexport.targetDeviceId.id,
          filename: $scope.liveexport.targetFilename + ".csv"
        }
      }, function() {
        $scope.liveexport.running = state;
      });
    }
  };

  $scope.dropdownSettings = {
    smartButtonMaxItems: 3,
    idProp: 'id',
    buttonClasses: "btn btn-info"
  };

  // Date picker elements
  $scope.$creationFrom = $("[name='creationFrom']");
  $scope.$creationTo = $("[name='creationTo']");

  // Configure date pickers
  var datePickerOptions = {
    clearBtn: true,
    todayBtn: "linked",
    autoclose: true,
    format: 'yyyy-mm-dd'
  };
  $scope.$creationFrom.datepicker(datePickerOptions);
  $scope.$creationTo.datepicker(datePickerOptions);

  $scope.$creationFrom.datepicker('setDate', startOfDay);
  $scope.$creationTo.datepicker('setDate', startOfDay);

  $scope.$creationFrom.on("changeDate", function(e) {
    $scope.filter.creationFrom = e.date ? e.date.valueOf() : null;
  });

  $scope.$creationTo.on("changeDate", function(e) {
    $scope.filter.creationTo = e.date ? e.date.valueOf() + 24*60*60*1000 : null;
  });

  $scope.normalizeTC = function() {
    TimecodeService.normalizeTC($scope.filter.timecodeFrom);
    TimecodeService.normalizeTC($scope.filter.timecodeTo);
  };

  $scope.refreshDevices = function() {
    var devices = _.filter(DeviceModel.devices, function(device) {
      return device.type == 'zeiss';
    });
    var dropdownOptions = _.map(devices, function(device) {
      return {
        id: device.id,
        label: device.name
      };
    });
    $scope.dropdownOptions.devices = dropdownOptions;

    $scope.dropdownOptions.storageDevices = _.chain(DeviceModel.devices).filter(function(device) {
      return device.type == 'storage' && device.connected;
    }).map(function(device) {
      return { id: device.id, text: device.name, type: 'USB storage', disabled: false };
    }).value();
    if ($scope.dropdownOptions.storageDevices.length == 0) {
      $scope.dropdownOptions.storageDevices.push({ id: -1, text: 'No USB storage available', type: 'USB storage', disabled: true });
    }
    $scope.dropdownOptions.usbDevices = $scope.dropdownOptions.storageDevices.slice(0);
    $scope.dropdownOptions.storageDevices.unshift({ id: 0, text: 'Download' });

    updateLiveExportUIFromChannelConfiguration();
  };

  $scope.runExport = function() {
    var fileName = $scope.exportTargetFilename || "lensdata";
    var deviceIds = _.map($scope.filter.devices, function(device) { return device.id; });

    if ($scope.exportTargetDeviceId.id == 0) {
      var deviceIdParam = deviceIds.join(",");
      var minTimecodeParam = $scope.formatTimecode($scope.filter.timecodeFrom);
      var maxTimecodeParam = $scope.formatTimecode($scope.filter.timecodeTo);
      var maxCreation = $scope.filter.creationTo + 24*3600000; // add 24h to end date

      var queryString = '?minTimestamp=' + $scope.filter.creationFrom +
          '&maxTimestamp=' + maxCreation +
          '&minTimecode=' + minTimecodeParam +
          '&maxTimecode=' + maxTimecodeParam +
          '&deviceIds=' + deviceIdParam +
          '&fileName=' + fileName + ".csv";

      location.href = '/lensdata' + queryString;
    } else {
      $scope.busy = true;

      var tcFrom = $scope.filter.timecodeFrom;
      var tcTo = $scope.filter.timecodeTo;
      var tcFromSeconds = parseInt(tcFrom.hours) * 3600 + parseInt(tcFrom.minutes) * 60 + parseInt(tcFrom.seconds);
      var tcToSeconds = parseInt(tcTo.hours) * 3600 + parseInt(tcTo.minutes) * 60 + parseInt(tcTo.seconds);

      var demand = {
        deviceIds: deviceIds,
        minTimestamp: $scope.filter.creationFrom,
        maxTimestamp: $scope.filter.creationTo,
        minTimecodeSeconds: tcFromSeconds,
        minTimecodeFrames: parseInt(tcFrom.frames),
        maxTimecodeSeconds: tcToSeconds,
        maxinTimecodeFrames: parseInt(tcTo.frames)
      };

      jQuery.ajax({
        type: 'POST',
        url: '/lensdata/' + $scope.exportTargetDeviceId.id + '/' + fileName + '.csv',
        data: JSON.stringify(demand),
        contentType: "application/json"
      }).success(function(data) {
        $scope.busy = false;
        $scope.$apply();
      });
    }
  };

  $scope.deleteAllLensData = function() {
    var answer = confirm("Do you really want to delete all lens data?");
    if (answer) {
      $scope.busy = true;

      jQuery.ajax({
        type: 'DELETE',
        url: '/lensdata/',
      }).success(function(data) {
        $scope.busy = false;
        $scope.$apply();
        alert("All lens data deleted");
      });
    }
  };

  $scope.formatTimecode = function(tc) {
    return tc.hours + ":" + tc.minutes + ":" + tc.seconds + ":" + tc.frames;
  };

  $scope.normalizeTC();
  $scope.refreshDevices();

  DeviceModel.addEventListener('change', function(type) {
    $scope.refreshDevices();
    $scope.$apply();
  });

  channelModel.addEventListener('change', function(type) {
    updateLiveExportUIFromChannelConfiguration();
  });
  updateLiveExportUIFromChannelConfiguration();

}]);
