angular.module('easyscott.live', []).controller('LiveController', [ '$scope', '$filter', 'WebsocketService', 'TimecodeService', 'DeviceModel', 'LiveClipModel', 'ClipMetadataTableCreator', function($scope, $filter, WebsocketService, TimecodeService, DeviceModel, LiveClipModel, ClipMetadataTableCreator) {
  $scope.devices = [];
  $scope.clips = [];
  
  $scope.refreshState = function() {
    $scope.refreshDevices();
    $scope.refreshClips();
  };

  $scope.refreshDevices = function() {
    $scope.devices = _.chain(DeviceModel.devices)
      .filter(function(device) {
        var deviceTypes = device.deviceTypes;
        if(deviceTypes.indexOf('CAMERA') >= 0 || deviceTypes.indexOf('SOUND_RECORDER') >= 0 || deviceTypes.indexOf('SLATE') >= 0) {
          return true;
        }
        return false;
      })
      .sortBy(function(device) {
        return device.name;
      })
      .value();
  };

  $scope.refreshClips = function() {
    // augment clips with source device name
    $scope.clips = _.map(LiveClipModel.clips, function(clip) {
      var sourceDevice = _.find($scope.devices, function(device) {
        return device.id == clip.sourceDeviceId;
      });

      if (sourceDevice) {
        clip.sourceDeviceName = sourceDevice.name;
      } else {
        clip.sourceDeviceName = clip.sourceDeviceId;
      }

      return clip;
    });
  };
  
  $scope.openMetadata = function(clip) {
    $('#clipMetadata').attr('data-clipId', clip.id);
    $('#clipMetadata').modal('show');
  };
  
  // metadata Modal Dialog
  $scope.$on('$viewContentLoaded', function() {
    $('#clipMetadata').on('show.bs.modal', function() {
      var $element = $(this);
      var $body = $element.find('.modal-body');
      var $table = $body.find('table');

      var clipId = $element.attr('data-clipId');
      var clip = _.filter($scope.clips, function(clip) {
        return clip.id == clipId;
      })[0];

      // Clean dialog
      $table.empty();
      $body.find('i').remove();
      $body.find('p').remove();

      // Show spinner
      $body.append($('<i>').addClass('fa fa-spinner fa-spin'));

      if (clip != null) {
        ClipMetadataTableCreator.createTable(clip, function(table) {
          $table.replaceWith(table);

          // Remove spinners
          $body.find('.fa-spinner').remove();
        });
      } else {
        $body.find('.fa-spinner').replaceWith($('<p>').text('Sorry, cannot get the clip metadata'));
      }
    })
  });

  DeviceModel.addEventListener('change', function(type) {
    $scope.refreshState();
    $scope.$apply();
  });

  LiveClipModel.addEventListener('change', function(type) {
    $scope.refreshState();
    $scope.$apply();
  });

  $scope.refreshState();
}]);
