angular.module('easyscott.slate', [])
    .controller('SlateController', ['$scope', 'WebsocketService', 'SlateModel', 'DeviceModel', function ($scope, WebsocketService, SlateModel, DeviceModel) {
      $scope.slateInfo = SlateModel.upcomingSlateInfo || {};
      $scope.lastReceivedSlateInfo = Object.assign({}, SlateModel.upcomingSlateInfo);
      $scope.editMode = false;
      $scope.minimal = true;

      $scope.slateTimecode = 0;
      $scope.lastSlateTimcodeSettingTimestamp = new Date();

      $scope.startEditMode = function () {
        $scope.editMode = true;
        $scope.setActions([
          {
            klass: 'glyphicon glyphicon-ok',
            action: function () {
              $scope.sendSlateInfo();
            }
          },
          {
            klass: 'glyphicon glyphicon-remove',
            action: function () {
              $scope.cancelEditing();
            }
          },
          {
            klass: 'glyphicon glyphicon-adjust',
            action: function () {
              $scope.setSlatingType($scope.slateInfo.slatingType === 'eu' ? 'us' : 'eu');
            }
          }
        ])
      };

      $scope.endEditMode = function () {
        $scope.editMode = false;
        $scope.setActions([{
          klass: 'glyphicon glyphicon-pencil',
          action: function () {
            $scope.startEditMode();
          }
        }]);
      };

      $scope.sendSlateInfo = function () {
        $scope.slateInfo.typedSlateInfo = null;
        WebsocketService.sendMessage({
          request: 'UPCOMING_SLATEINFO',
          slateInfo: $scope.slateInfo,
        });
        $scope.endEditMode();
      };

      $scope.cancelEditing = function () {
        $scope.slateInfo = $scope.lastReceivedSlateInfo || {};
        $scope.endEditMode();
      };

      $scope.integrateSlateInfo = function () {
        if (!$scope.slateInfo.rolls) {
          return;
        }
        const rolls = Object.values($scope.slateInfo.rolls);
        if (rolls.length > 0) {
          $scope.slateInfo.camRoll = rolls[0];
        }
      };

      $scope.sendClap = function () {
        if (!$scope.clapActive && !$scope.editMode) {
          $scope.clapActive = true;
          WebsocketService.sendMessage({
            request: 'CLAP',
          });
          window.setTimeout(jQuery.proxy(function () {
            this.clapActive = false;
            this.$apply();
          }, $scope), 5500);
        }
      };

      $scope.isUs = function () {
        return $scope.slateInfo.slatingType === 'us';
      };

      $scope.addSlate = function (n) {
        $scope.slateInfo.slate = $scope.addToString($scope.slateInfo.slate, n);
      };

      $scope.addTake = function (n) {
        $scope.slateInfo.take = $scope.addToString($scope.slateInfo.take, n);
      };

      $scope.addUsSlate = function(n) {
        const len = $scope.slateInfo.slate.length;
        const letter = $scope.slateInfo.slate.charAt(len - 1);
        if (letter >= "A" && letter <= "Z") {
          const nextLetter = String.fromCharCode(letter.charCodeAt(0) + n);
          if (nextLetter >= "A" && nextLetter <= "Z") {
            $scope.slateInfo.slate = $scope.slateInfo.slate.substr(0, len - 1) + nextLetter;
          }
        }
      };

      $scope.addToString = function(s, n) {
        const v = parseInt(s, 10);
        if (!isNaN(v)) {
          return (v + n).toString();
        } else {
          return s;
        }
      };

      $scope.setSlatingType = function(type) {
        $scope.slateInfo.slatingType = type;
      };

      $scope.formattedDate = function() {
        if (!$scope.slateInfo.date) { return ""; }
        const date = new Date($scope.slateInfo.date);
        if (!isNaN(date.valueOf())) {
          if (DeviceModel.getMasterclockDevice()) {
            const userbits = DeviceModel.getMasterclockDevice().properties.userbits;
            const lastByte = userbits.substr(6, 2);
            if (lastByte === 'AD') {
              return leftPad((date.getMonth() + 1).toString(), 2, "0") + "/" +
                  leftPad(date.getDate().toString(), 2, "0") + "/" +
                  date.getFullYear();
            } else if (lastByte === 'ED') {
              return leftPad(date.getDate().toString(), 2, "0") + "." +
                  leftPad((date.getMonth() + 1).toString(), 2, "0") + "." +
                  date.getFullYear();
            }
          }
        }
        return $scope.slateInfo.date;
      };

      var leftPad = function(s, n, c) {
        while (s.length < n) {
          s = c + s;
        }
        return s;
      };

      $scope.setFullscreen(true);
      $scope.endEditMode();
      $scope.integrateSlateInfo();

      // Double touch recognition

      $scope.handleClick = function (ev) {
        const lastEv = $scope.lastClickEvent;
        if (!lastEv) {
          $scope.lastClickEvent = ev;
          return;
        }

        const timeDiff = ev.timeStamp - lastEv.timeStamp;
        const distance = Math.sqrt(Math.pow(Math.abs(ev.clientX - lastEv.clientX), 2) + Math.pow(Math.abs(ev.clientY - lastEv.clientY), 2));

        console.log(timeDiff);
        console.log(distance);

        if (timeDiff < 300 && distance < 30) {
          $scope.editMode = !$scope.editMode;
        }

        $scope.lastClickEvent = ev;
      };

      SlateModel.addEventListener('change', function (type) {
        if (!$scope.editMode) {
          $scope.slateInfo = SlateModel.upcomingSlateInfo || {};
          $scope.integrateSlateInfo();
        }
        $scope.lastReceivedSlateInfo = Object.assign({}, SlateModel.upcomingSlateInfo);
        $scope.$apply();
      });

      // timecode
      WebsocketService.addEventListener('message', function (type, message) {
        if (message.request == 'TIMECODE') {
          if ($scope.slateTimecode != message.timecode.seconds && $scope.clapActive == false) {
            $scope.slateTimecode = message.timecode.seconds;
            $scope.lastSlateTimcodeSettingTimestamp = new Date();
          }

          $scope.$apply();
        }
      });

      $scope.$on('$destroy', function () {
        $scope.setFullscreen(false);
        $scope.setActions([]);
      });
    }]);
