angular.module('easyscott.update', [])
  .controller('UpdateController', ['$scope', function ($scope) {

    $scope.versions = null;
    $scope.uploadRunning = false;
    $scope.uploadPercent = 0;

    $scope.uploadUpgrade = function (e) {
      e.preventDefault();

      if (!$scope.file) {
        $scope.clearAlerts();
        $scope.addAlert('danger', 'Please select a file to upload');
        return;
      }

      if (!$scope.file.name.match(/\.ml-upgrade$/)) {
        $scope.clearAlerts();
        $scope.addAlert('danger', 'The update file must end with ".ml-upgrade"');
        return;
      }

      $scope.uploadFile($scope.file);
    };

    $scope.uploadFile = function (file) {
      $scope.clearAlerts();
      $scope.uploadRunning = true;

      var data = new FormData();
      data.append("file", file);

      $.ajax({
        url: '/upload/',
        type: 'POST',
        data: data,
        cache: false,
        processData: false,
        contentType: false,
        xhr: function () {
          var xhr = new window.XMLHttpRequest();
          xhr.upload.addEventListener('progress', $scope.updateProgress, false);
          return xhr;
        },
        success: function (data) {
          $scope.clearAlerts();
          $scope.addAlert('success', data);

          $scope.setEstimatedReconnectTimestamp(new Date().valueOf() + 5 * 60 * 1000);

          // Hack to grant a grace period for the connection overlay.
          $("#loading-overlay").css("display", "none");
          setTimeout(function () {
            $("#loading-overlay").css("display", "");
          }, 2000);
        },
        error: function (xhr) {
          $scope.clearAlerts();
          $scope.addAlert('danger', 'An error occured while uploading the file: ' + xhr.responseText);
        }
      });
    };

    $scope.updateProgress = function (event) {
      if (event.lengthComputable) {
        $scope.uploadPercent = parseInt(event.loaded / event.total * 100);
        $scope.$apply();
      }
    };

    // Configure the drop zone
    $(document).ready(function () {
      $(".fileDropZone").on("dragover", function (e) {
        e.preventDefault();
        $(this).css('opacity', 1);
        return false;
      });
      $(".fileDropZone").on("dragleave", function () {
        $(this).css('opacity', 0.3);
      });
      $(".fileDropZone").on("drop", function (e) {
        e.preventDefault();

        $scope.file = e.originalEvent.dataTransfer.files[0];
        $scope.uploadUpgrade(e);
        $(this).css('opacity', 0.3);
        return false;
      });
    });

    jQuery.ajax({
      url: '/version/'
    })
      .success(jQuery.proxy(function (data) {
        $scope.versions = data;
        $scope.$apply();
      }, this));

  }])

  .directive("fileread", [function () {
    return {
      scope: {
        fileread: "="
      },
      link: function (scope, element, attributes) {
        element.bind("change", function (changeEvent) {
          scope.$apply(function () {
            scope.fileread = changeEvent.target.files[0];
          });
        });
      }
    }
  }]);
