angular.module('easyscott.wifi', []).controller('WifiController', [ '$scope', 'WebsocketService', 'SystemPropertiesModel', function($scope, WebsocketService, SystemPropertiesModel) {
  $scope.wifi = { wifi_ssid: '', channel: "1" };
  $scope.ssidList = null;
  $scope.state = null;
  $scope.joinWifi = { ssid: '', psk: '' };

  $scope.renameWifi = function() {
    if($scope.wifi.wifi_ssid.length < 8) {
      $scope.addAlert('danger', 'New WiFi name must be at least 8 characters.');
      return;
    }

    var message = {
      request: 'CONFIGURE_WIFI',
      ssid: $scope.wifi.wifi_ssid,
      channel: $scope.wifi.channel
    };

    $scope.executeSynchronousRequest(message, function(response) {
      if (response == null || response.success === false) {
        $scope.addAlert('danger', 'Error while reconfiguring WiFi');
      } else {
        $scope.addAlert('success', 'WiFi will be reconfigured. Please connect to the following WiFi: "' + $scope.wifi.wifi_ssid + '". It may take some time until it appears in your WiFi list.');
      }
    });
  };
  
  $scope.joinWifi = function() {
    if ($scope.joinWifi.psk.length < 8) {
      $scope.addAlert('danger', 'WiFi password must be at least 8 characters long');
      return;
    }
    var message = {
      request: 'CONFIGURE_CLIENT_WIFI',
      ssid: $scope.joinWifi.ssid,
      psk: $scope.joinWifi.psk
    }
    $scope.executeSynchronousRequest(message, function(response) {
      if (response == null || response.success === false) {
        $scope.addAlert('danger', 'Error while reconfiguring WiFi');
      } else {
        $scope.addAlert('success', 'WiFi will be reconfigured. You can reset the WiFi settings via the OLED menu.');
      }
    });
  };

  $scope.requestSsids = function () {
    $scope.ssidList = null;
    message = { request: 'REQUEST_WIFI_SSIDS' };
    WebsocketService.sendMessage(message);
  }

  $scope.requestState = function() {
    $scope.state = null;
    message = { request: 'REQUEST_WIFI_STATE' };
    WebsocketService.sendMessage(message);
  }

  $scope.requestSystemProperties = function() {
    var message = { request: 'REQUEST_SYSTEM_PROPERTIES' };
    WebsocketService.sendMessage(message);
  };

  SystemPropertiesModel.addEventListener('change', function(type) {
    if (SystemPropertiesModel.properties.wifi_ssid) {
      if ($scope.wifi.wifi_ssid !== SystemPropertiesModel.properties.wifi_ssid) {
        $scope.wifi.wifi_ssid = SystemPropertiesModel.properties.wifi_ssid;
      }
      $scope.wifi.channel = SystemPropertiesModel.properties.wifi_channel + "";
    }
    $scope.$apply();
  });

  WebsocketService.addEventListener('message', function(type, message) {
    if (message.request === 'WIFI_SSID_LIST') {
      $scope.ssidList = message.ssidList;
      $scope.joinWifi.ssid = message.ssidList[0];
      $scope.$apply();
    } else if (message.request === 'WIFI_STATE') {
      $scope.state = message.state;
      $scope.$apply();
    }
  });

  $scope.requestSystemProperties();
  $scope.requestSsids();
  $scope.requestState();
}]);
