var ChannelModel = EventDispatcher.extend({
  started: false,
  channels: [],
  service: null,

  start: function() {
    if(!this.started) {
      this.service.addEventListener('message', jQuery.proxy(function(type, message) {
        if(message.request == "CHANNEL_STATE") {
          this.updateOrInsert(message);
        }
      }, this));

      this.started = true;
    }
  },
  updateOrInsert: function(newChannel) {
    if(newChannel.removed == false) {
      var inserted = false;

      jQuery.each(this.channels, function(idx, channel) {
        if(channel.id == newChannel.id) {
          // replace object
          jQuery.each(newChannel, function(idx, item) {
            channel[idx] = item;
          });

          inserted = true;
        }
      });

      if(!inserted) {
        this.channels.push(newChannel);
      }
      this.dispatchEvent('change');
    } else {
      var newChannels = [];
      jQuery.each(this.channels, function(idx, channel) {
        if(channel.id != newChannel.id) {
          newChannels.push(channel);
        }
      });
      this.channels = newChannels;
      this.dispatchEvent('change');
    }
  },
  getActiveACNChannelId: function() {
    var channelId = null;

    jQuery.each(this.channels, function(idx, channel) {
      if(channel.type == 'acn') {
        channelId = channel.id;
      }
    });

    return channelId;
  },
  getActiveCLinkChannelId: function() {
    var channelId = null;

    jQuery.each(this.channels, function(idx, channel) {
      if(channel.type == 'clink') {
        channelId = channel.id;
      }
    });

    return channelId;
  },
  getActiveZeissChannelId: function() {
    var channelId = null;

    jQuery.each(this.channels, function(idx, channel) {
      if(channel.type == 'zeiss') {
        channelId = channel.id;
      }
    });

    return channelId;
  },
  isACNGenerator: function() {
    var result = false;

    jQuery.each(this.channels, function(idx, channel) {
      if(channel.type == 'acn') {
        if(channel.configuration && channel.configuration.generatorMode == 'true') {
          result = true;
        }
      }
    });

    return result;
  },
  getChannelById: function(id) {
    return _.find(this.channels, function(c) {
      return c.id == id;
    });
  }
});

(function () {
  var ChannelModelProvider = Class.extend({
    instance: new ChannelModel(),
    $get:['WebsocketService', function(websocketService) {
      this.instance.service = websocketService;
      return this.instance;
    }]
  })
  angular.module('easyscott.ChannelModel', ['easyscott.WebsocketService']).provider('ChannelModel', ChannelModelProvider);
}());
