var ClipModel = EventDispatcher.extend({
  $http: null,
  started: false,
  allClips: [],
  clips: [],
  total: 0,
  filtered: 0,
  service: null,
  filter: null,
  clipFetchInProgress: false,
  clipFetchRequested: false,
  currentPage: 0,
  pageSize: 12,
  currentViewId: '',

  start: function() {
    if(!this.started) {
      this.service.addEventListener('message', jQuery.proxy(function(eventType, message) {
        if(message.request == 'CLIP_STATE') {
          if (message.removed) {
            this.dispatchEvent('clipRemoved', message.id);
          } else {
            this.dispatchEvent('clipChanged', message.type, message.id, message.clip);
          }
        }
      },this));
      this.fetchClips();
      this.started = true;
    }
  },
  fetchClips: function() {
    if (this.clipFetchInProgress) {
      this.clipFetchRequested = true;
      return;
    }

    if(!this.filter) {
      this.filter = {};
    }
    this.filter.from = this.currentPage * this.pageSize;
    this.filter.to = (this.currentPage + 1) * this.pageSize;
    this.filter.v = this.currentViewId;
    
    if((this.filtered == 0 && this.currentPage == 0) || this.filter.from <= this.filtered) {
      this.clipFetchRequested = false;
      this.clipFetchInProgress = true;
      this.dispatchEvent('clipFetchStarted');
      
      jQuery.ajax({
        url: '/clips/',
        data: this.filter
      })
        .success(jQuery.proxy(this.receiveClips, this))
        .complete(jQuery.proxy(function() {
          this.clipFetchInProgress = false;
          this.dispatchEvent('clipFetchFinished');

          if (this.clipFetchRequested) {
            this.fetchClips();
          }
        }, this));
    } else {
      this.clipFetchInProgress = false;
      this.dispatchEvent('clipFetchFinished');
    }
  },
  receiveClips: function(newClips) {
    var targetClips = newClips.clips;
    if(this.currentPage > 0) {
      targetClips = this.clips.concat(targetClips);
    }
    
    this.clipFetchInProgress = false;

    this.clips = this.helper.mergeArrays(this.clips, targetClips, function(a, b) {
      return a && b && a.id == b.id;
    });
    
    // remove duplicates
    this.clips = this.removeDuplicateIds(this.clips);

    // TODO: dirty hack - remove in future versions
    if (!this.allClips || this.allClips.length < this.clips.length) {
      this.allClips = _.cloneDeep(this.clips);
    }

    this.total = newClips.total;
    this.filtered = newClips.filtered;
    this.dispatchEvent('change');
  },
  setFilter: function(filter) {
    if (this.filter != filter) {
      this.filter = filter;
      this.currentPage = 0;
      this.total = 0;
      this.filtered = 0;
      this.currentViewId = this.generateRandomViewId();
      this.fetchClips();
    }
  },
  loadNextPage: function() {
    this.currentPage++;
    this.fetchClips();
  },
  removeDuplicateIds: function(list) {
    var filtered = [];
    var keys = [];
    
    angular.forEach(list, function(item) {
      var key = item.id;
      if(keys.indexOf(key) === -1) {
        keys.push(key);
        filtered.push(item);
      }
    });
    
    return filtered;
  },
  generateRandomViewId: function() {
    var viewId = '';
    var possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_-.';

    for(var i = 0; i < 8; i++) {
      viewId += possible.charAt(Math.floor(Math.random() * possible.length));
    }

    return viewId;
  }
});

(function () {
  var ClipModelProvider = Class.extend({
    instance: new ClipModel(),
    $get:['$http', 'WebsocketService', 'HelperService', function($http, websocketService, helperService) {
      this.instance.$http = $http;
      this.instance.service = websocketService;
      this.instance.helper = helperService;
      return this.instance;
    }]
  })
  angular.module('easyscott.ClipModel', ['easyscott.WebsocketService', 'easyscott.HelperService']).provider('ClipModel', ClipModelProvider);
}());
