var LiveClipModel = EventDispatcher.extend({
  $http: null,
  started: false,
  clips: [],
  service: null,
  clipFetchInProgress: false,

  start: function() {
    if(!this.started) {
      this.service.addEventListener('message', jQuery.proxy(function(type, message) {
        if(message.request == "CLIP_STATE") {
          this.updateOrInsert(message);
        }
      }, this));

      this.fetchClips();
      this.started = true;
    }
  },
  fetchClips: function() {
    this.clipFetchInProgress = true;
    this.dispatchEvent('clipFetchStarted');

    jQuery.ajax({
      url: '/clips/',
      data: {
        from: 0,
        to: 100
      }
    }).success(jQuery.proxy(this.receiveClips, this))
      .complete(jQuery.proxy(function() {
        this.clipFetchInProgress = false;
        this.dispatchEvent('clipFetchFinished');
      }, this));
  },
  updateOrInsert: function(newClip) {
    this.clips = _.filter(this.clips, function(clip) {
      return clip.id != newClip.id;
    });
    
    if (newClip.removed === true) {
      this.dispatchEvent('change', [newClip.id]);
      this.dispatchEvent('clipUpdate', [newClip.id]);
    } else {
      if (newClip.clip) {
        this.clips.push(newClip.clip);
      }
      this.dispatchEvent('change', [newClip.clip.id]);
      this.dispatchEvent('clipUpdate', [newClip.clip.id]);
    }
  },
  setDemand: function(demand) {
    this.service.sendMessage({request: 'CLIP_DEMAND', demand: demand});
  },
  receiveClips: function(newClips) {
    this.clipFetchInProgress = false;

    this.clips = this.helper.mergeArrays(this.clips, newClips.clips, function(a, b) {
      return a && b && a.id == b.id;
    });

    this.dispatchEvent('change', _.map(newClips.clips, function(clip) {
      return clip.id;
    }));
  },
});

(function () {
  var LiveClipModelProvider = Class.extend({
    instance: new LiveClipModel(),
    $get:['$http', 'WebsocketService', 'HelperService', function($http, websocketService, helperService) {
      this.instance.$http = $http;
      this.instance.service = websocketService;
      this.instance.helper = helperService;
      return this.instance;
    }]
  })
  angular.module('easyscott.LiveClipModel', ['easyscott.WebsocketService', 'easyscott.HelperService']).provider('LiveClipModel', LiveClipModelProvider);
}());
