var CjamSettingsService = angular.module('easyscott.CjamSettingsService', []);

CjamSettingsService.factory('CjamSettingsService', ['$rootScope', 'WebsocketService', 'ChannelModel', 'DeviceModel', function($rootScope, WebsocketService, ChannelModel, DeviceModel) {
  return {
    framerateOptions: [
      { value: '0', text: '23.976F' },
      { value: '1', text: '24F' },
      { value: '2', text: '25F' },
      { value: '3', text: '29.97F' },
      { value: '5', text: '29.97DF' },
      { value: '4', text: '30F' }
    ],

    projectFramerateOptions: [
      { value: 'FPS_23_98', text: '23.976' },
      { value: 'FPS_24', text: '24' },
      { value: 'FPS_25', text: '25' },
      { value: 'FPS_29_97', text: '29.97' },
      { value: 'FPS_29_97_DROP', text: '29.97 Drop' },
      { value: 'FPS_30', text: '30' },
      // { value: 'FPS_30_DROP', text: '30 Drop' },
      { value: 'FPS_47_95', text: '47.952' },
      { value: 'FPS_48', text: '48' },
      { value: 'FPS_50', text: '50'},
      { value: 'FPS_59_94', text: '59.94' },
      { value: 'FPS_59_94_DROP', text: '59.94 Drop' },
      { value: 'FPS_60', text: '60' },
      // { value: 'FPS_60_DROP', text: '60 Drop' }
    ],

    channelOptions: [
      { value: '10', text: 'OFF' },
      { value: '11', text: '11' },
      { value: '12', text: '12' },
      { value: '13', text: '13' },
      { value: '14', text: '14' },
      { value: '15', text: '15' },
      { value: '16', text: '16' },
      { value: '17', text: '17' },
      { value: '18', text: '18' },
      { value: '19', text: '19' },
      { value: '20', text: '20' },
      { value: '21', text: '21' },
      { value: '22', text: '22' },
      { value: '23', text: '23' },
      { value: '24', text: '24' },
      { value: '25', text: '25' },
      { value: '26', text: '26' }
    ],

    setCjamMaster: function(isMaster) {
      if (isMaster && DeviceModel.getMasterclockDevice().properties.acnDisplayMode === "CJAM_SLAVE") {
        // We are jammed => forbid switching mode
        $('#masterModeForbidden').modal('show');
        return;
      }

      var command = isMaster ? "STARTCJAM" : "STOPCJAM";
      var message =  {
        request: 'CHANNEL_COMMAND',
        id: ChannelModel.getActiveACNChannelId(),
        command: command
      };

      WebsocketService.sendMessage(message);

      if ($rootScope.generator === isMaster) {
        $rootScope.changingMode = true;
      }
    },

    setProjectFramerate: function(projectFramerate) {
      var channelId = ChannelModel.getActiveACNChannelId();
      if (channelId) {
        WebsocketService.sendMessage({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SET_PROJECT_FRAMERATE',
          arguments: {
            fps: projectFramerate
          }
        });
      }
    },

    setChannel: function(channel) {
      var channelId = ChannelModel.getActiveACNChannelId();
      if (channelId) {
        WebsocketService.sendMessage({
          request: 'CHANNEL_COMMAND',
          id: channelId,
          command: 'SETCHANNEL',
          arguments: {
            channel: channel
          }
        });
      }
    },

    updateCjamStatus: function(cjamStatus) {
      var masterClock = DeviceModel.getMasterclockDevice();
      if (masterClock) {
        cjamStatus.framerate = _.chain(this.framerateOptions).filter(function(i) {
          return i.value === masterClock.properties.lockitFramerate;
        }).first().value();

        cjamStatus.projectFramerate = _.chain(this.projectFramerateOptions).filter(function(i) {
          return i.value === masterClock.properties.projectFramerate;
        }).first().value();

        cjamStatus.channel = _.chain(this.channelOptions).filter(function(i) {
          return i.value === masterClock.properties.acnChannel;
        }).first().value();

        cjamStatus.cjamRunning = masterClock.properties.cjamEnabled === "true";
        cjamStatus.timecodeSet = parseInt(masterClock.properties.generatorSource) !== "ZERO";

        var userbits = masterClock.properties.userbits;
        if (userbits) {
          userbits = userbits.toUpperCase();
          cjamStatus.userbits = {
            one: userbits.substr(0, 2),
            two: userbits.substr(2, 2),
            three: userbits.substr(4, 2),
            four: userbits.substr(6, 2)
          };
        }

        cjamStatus.tunePwm = masterClock.properties.tunePWM;
      }
    }
  };
}]);
