var HelperService = function() {};

// Returns newArray while keeping the $$hashCode property of the corresponding
// element in oldArray. The provided isEqual function is used for comparing.
HelperService.prototype.mergeArrays = function(oldArray, newArray, isEqual) {
  if (typeof isEqual !== 'function') {
    isEqual = _.isEqual;
  }

  return _.map(newArray, function(clip) {
    var existingClip = _.find(oldArray, function(oldClip) {
      return isEqual(oldClip, clip);
    });
    if (existingClip) {
      clip.$$hashKey = existingClip.$$hashKey;
    }
    return clip;
  });
};

// Returns a lodash instance with a modified indexOf function which uses the
// provided isEqual function for comparison.
HelperService.prototype.lodashWithEquality = function(isEqual) {
  return _.runInContext().mixin({
    indexOf: function(haystack, needle) {
      for (var i = 0; i < haystack.length; i++) {
        var candidate = haystack[i];
        if (isEqual(candidate, needle)) {
          return i;
        }
      }
      return -1;
    }
  });
};

// Formats the given date in ISO form: yyyy-mm-dd
// If no date is provided the current date is used.
HelperService.prototype.isoDate = function(date) {
    if (!date) {
        date = new Date();
    }

    var year = date.getFullYear();
    var month = date.getMonth() + 1;
    if (month < 10) {
        month = "0" + month;
    }
    var day = date.getDate();
    if (date < 10) {
        date = "0" + date;
    }

    return year + "-" + month + "-" + day;
};

(function () {
  var HelperServiceProvider = Class.extend({
    instance: new HelperService(),
    $get: [function() {
      return this.instance;
    }]
  });
  angular.module('easyscott.HelperService',[]).provider('HelperService',HelperServiceProvider);
}());
