var LtcService = function() {};

(function () {
  var LtcServiceProvider = Class.extend({
    instance: new LtcService(),
    $get: [function() {
      return this.instance;
    }]
  });
  angular.module('easyscott.LtcService',[]).provider('LtcService', LtcServiceProvider);
}());

function serializeTc(tc, userbits, buffer) {
  buffer[0] = ((bcdEncode(tc.frames % 10, 4)) << 4) | ((userbits[0] || 0) & 0xF);
  buffer[1] = ((bcdEncode(Math.floor(tc.frames / 10), 2)) << 6) | ((userbits[1] || 0) & 0xF);
  buffer[2] = ((bcdEncode(tc.seconds % 10, 4)) << 4) | ((userbits[2] || 0) & 0xF);
  buffer[3] = ((bcdEncode(Math.floor(tc.seconds / 10), 3)) << 5) | ((userbits[3] || 0) & 0xF);
  buffer[4] = ((bcdEncode(tc.minutes % 10, 4)) << 4) | ((userbits[4] || 0) & 0xF);
  buffer[5] = ((bcdEncode(Math.floor(tc.minutes / 10), 3)) << 5) | ((userbits[5] || 0) & 0xF);
  buffer[6] = ((bcdEncode(tc.hours % 10, 4)) << 4) | ((userbits[6] || 0) & 0xF);
  buffer[7] = ((bcdEncode(Math.floor(tc.hours / 10), 2)) << 6) | ((userbits[7] || 0) & 0xF);
  buffer[8] = 0x3F;
  buffer[9] = 0xFD;
}

LtcService.prototype.serializeTc = serializeTc;

// Data Structures

var LtcStreamer = function(tcMillis, fps, sampleRate) {
  this.tcMillis = tcMillis;
  this.fps = fps;
  this.sampleRate = sampleRate;
  this.tcFrameSampleLength = sampleRate / fps;
  this.reference = new Date().valueOf();
  this.encoder = new LtcEncoder(new ArrayBuffer(48000), 0, fps, 48000);
  this.tcFrameBuffer = new ArrayBuffer(10);
  this.rest = new ArrayBuffer(48000);
  this.restLength = 0;
  this.lastTc = null;
};

LtcStreamer.prototype.currentTc = function() {
  var offset = new Date().valueOf() - this.reference;
  var millis = this.tcMillis + offset;
  return new Timecode(millis * this.fps / 1000, this.fps);
};

LtcStreamer.prototype.fill = function(buffer) {
  var remaining = buffer.length;
  var restLength = Math.min(remaining, this.restLength);
  for (var i = 0; i < restLength; i++) {
    buffer[i] = this.rest[i];
  }

  remaining -= restLength;
  if (remaining === 0) {
    // There is still rest left
    var newRestLength = this.rest.length - restLength;
    for (var i = 0; i < newRestLength; i++) {
      this.rest[i] = this.rest[restLength + i];
    }
    this.restLength = newRestLength;
    return;
  }

  var nFrames = Math.ceil(remaining / this.tcFrameSampleLength);
  this.encoder.offset = 0;
  var tc = this.lastTc || this.currentTc();
  for (var i = 0; i < nFrames; i++) {
    serializeTc(tc.getObj(), [], this.tcFrameBuffer);
    for (var j = 0; j < 80; j++) {
      this.encoder.writeBit(extractBit(this.tcFrameBuffer, j));
    }
    tc.totalFrames += 1;
  }
  this.lastTc = tc;

  var samplesWritten = nFrames * this.tcFrameSampleLength;
  var newRestLength = samplesWritten - remaining;

  for (var i = 0; i < remaining; i++) {
    buffer[i + restLength] = this.encoder.buffer[i];
  }
  for (var i = 0; i < newRestLength; i++) {
    this.rest[i] = this.encoder.buffer[remaining + i];
  }
  this.restLength = newRestLength;
};

var LtcEncoder = function(buffer, offset, fps, sampleRate) {
  this.buffer = buffer;
  this.offset = offset;
  this.fps = fps;
  this.sampleRate = sampleRate;
  this.lastHigh = false;
  this.bitPeriod = sampleRate / (fps * 80);
  this.volume = 90;
};

LtcEncoder.prototype.writeBit = function(isHigh) {
  var initLevel = this.lastHigh ? -this.volume : this.volume;
  if (isHigh) {
    var halfPeriod = Math.floor(this.bitPeriod / 2);
    for (var i = 0; i < halfPeriod; i++) {
      this.buffer[this.offset + i] = initLevel;
    }
    for (var i = halfPeriod; i < this.bitPeriod; i++) {
      this.buffer[this.offset + i] = -initLevel;
    }
  } else {
    for (var i = 0; i < this.bitPeriod; i++) {
      this.buffer[this.offset + i] = initLevel;
    }
    this.lastHigh = !this.lastHigh;
  }
  this.offset += this.bitPeriod;
};

var Timecode = function(totalFrames, fps) {
  this.totalFrames = totalFrames;
  this.fps = fps;
};

Timecode.prototype.totalSeconds = function() {
  return this.totalFrames / this.fps;
};

Timecode.prototype.hours = function() {
  return Math.floor(this.totalSeconds() / 3600);
};

Timecode.prototype.minutes = function() {
  return Math.floor((this.totalSeconds() / 60) % 60);
};

Timecode.prototype.seconds = function() {
  return Math.floor(this.totalSeconds() % 60);
};

Timecode.prototype.fs = function() {
  return Math.floor(this.totalFrames % this.fps);
};

Timecode.prototype.getObj = function() {
  return {
    hours: this.hours(),
    minutes: this.minutes(),
    seconds: this.seconds(),
    frames: this.fs()
  }
};

LtcService.prototype.Timecode = Timecode;
LtcService.prototype.LtcEncoder = LtcEncoder;
LtcService.prototype.LtcStreamer = LtcStreamer;

// Helper functions

function bcdEncode(value, bits) {
  var result = 0;
  for (var i = 0; i < bits; i++) {
    result |= ((value >> i) & 0x1) << (bits - 1 - i);
  }
  return result;
}

function extractBit(buffer, n) {
  var byteN = n >> 3;
  var bitN = n & 7;
  var mask = 1 << (7 - bitN);
  return (buffer[byteN] & mask) !== 0;
}
