var TimecodeService = function() {};

TimecodeService.prototype.fromTotalSeconds = function(totalSeconds) {
  return {
    hours: Math.floor(totalSeconds / 3600),
    minutes: Math.floor((totalSeconds / 60) % 60),
    seconds: totalSeconds % 60,
    frames: 0
  };
};

TimecodeService.prototype.parse = function(input) {
  var m = input.matchAll(/(\d{2}):(\d{2}):(\d{2}):(\d{2})/).next().value;
  if (!m) { return null; }
  return {
    hours: parseInt(m[1], 10),
    minutes: parseInt(m[2], 10),
    seconds: parseInt(m[3], 10),
    frames: parseInt(m[4], 10),
  };
};

TimecodeService.prototype.fromNow = function() {
  var currentDate = new Date();
  return {
    hours: currentDate.getHours(),
    minutes: currentDate.getMinutes(),
    seconds: currentDate.getSeconds(),
    frames: Math.round(currentDate.getMilliseconds() * 24 / 1000)
  };
};

TimecodeService.prototype.formatTimeCodeComponent = function(n, maxValue) {
  maxValue = maxValue || 59;
  n = parseInt(n);

  if (isNaN(n)) {
    n = 0;
  } else if (n > maxValue) {
    n = maxValue;
  } else if (n < 0) {
    n = 0;
  }
  return ((n < 10) ? "0" : "") + n
}

TimecodeService.prototype.normalizeTC = function(timecode) {
  timecode.hours = this.formatTimeCodeComponent(timecode.hours, 23);
  timecode.minutes = this.formatTimeCodeComponent(timecode.minutes);
  timecode.seconds = this.formatTimeCodeComponent(timecode.seconds);
  timecode.frames = this.formatTimeCodeComponent(timecode.frames);

  return timecode;
};

TimecodeService.prototype.render = function(timecode) {
  var normalizedTC = this.normalizeTC(_.clone(timecode));

  return normalizedTC.hours
    + ":" + normalizedTC.minutes
    + ":" + normalizedTC.seconds
    + ":" + normalizedTC.frames;
};

TimecodeService.prototype.difference = function(a, b) {
  var aSecs = a.seconds + a.minutes * 60 + a.hours * 3600;
  var bSecs = b.seconds + b.minutes * 60 + b.hours * 3600;
  return aSecs - bSecs;
};

(function () {
  var TimecodeServiceProvider = Class.extend({
    instance: new TimecodeService(),
    $get: [function() {
      return this.instance;
    }]
  });
  angular.module('easyscott.TimecodeService',[]).provider('TimecodeService',TimecodeServiceProvider);
}());
