var WebsocketService = EventDispatcher.extend({
  started: false,
  connected: false,
  connecting: false,
  wasConnected: false,
  $scope: null,
  socketConnection: null,

  start: function() {
    if(!this.started) {
      this.connect();

      setInterval(jQuery.proxy(function() {
        this.sendMessage(JSON.stringify({request:'PING'}));
      }, this), 10000);

      this.started = true;
    }
  },
  connect: function() {
    if (!this.connected && !this.connecting) {
      this.connecting = true;

      this.socketConnection = new WebSocket('ws://' + window.location.host + '/api/');
      this.socketConnection.onopen = jQuery.proxy(this.onOpen, this);
      this.socketConnection.onmessage = jQuery.proxy(this.onMessage, this);
      this.socketConnection.onclose = jQuery.proxy(this.onClose, this);
      this.socketConnection.onerror = jQuery.proxy(this.onClose, this);
    }
  },
  onOpen: function() {
    if (this.wasConnected) {
      window.location.reload();
      return;
    }

    this.connected = true;
    this.wasConnected = true;
    this.connecting = false;
    this.dispatchEvent('connect');

    this.sendMessage(JSON.stringify({request: 'REQUEST_CHANNELS'}));
    this.sendMessage(JSON.stringify({request: 'REQUEST_DEVICES'}));
    this.sendMessage(JSON.stringify({request: 'REQUEST_SYSTEM_PROPERTIES'}));
  },
  onClose: function() {
    this.connected = false;
    this.connecting = false;
    this.dispatchEvent('disconnect');

    setTimeout(jQuery.proxy(function() {
      this.connect();
    }, this), 2000);
  },
  sendMessage: function(message, callback) {
    if(typeof message == 'object') {
      message = JSON.stringify(message);
    }

    this.waitForSocketConnection(jQuery.proxy(function() {
      this.socketConnection.send(message);
      if (typeof callback !== 'undefined') {
        callback();
      }
    }, this));
  },
  waitForSocketConnection: function(callback, interval) {
    if (this.socketConnection.readyState === 1) {
      callback();
    } else {
      setTimeout(jQuery.proxy(function () {
        this.waitForSocketConnection(callback);
      }, this), interval);
    }
  },
  onMessage: function(messageData) {
    var object = JSON.parse(messageData.data);
    this.dispatchEvent('message', object);
  }
});

(function () {
  var WebsocketServiceProvider = Class.extend({
    instance: new WebsocketService(),
    $get: [function() {
      return this.instance;
    }]
  });
  angular.module('easyscott.WebsocketService',[]).provider('WebsocketService',WebsocketServiceProvider);
}());
