var easyscott = angular.module('easyscott', [
  'ui.router',
  'ngAnimate',
  'ngSanitize',
  'LocalStorageModule',
  'xeditable',
  'infinite-scroll',
  'angularjs-dropdown-multiselect',
  'easyscott.filters',
  'easyscott.DeviceModel',
  'easyscott.ChannelModel',
  'easyscott.ClipModel',
  'easyscott.LiveClipModel',
  'easyscott.SlateModel',
  'easyscott.SystemPropertiesModel',
  'easyscott.CjamSettingsService',
  'easyscott.ClipMetadataTableCreator',
  'easyscott.HelperService',
  'easyscott.LtcService',
  'easyscott.ReportService',
  'easyscott.TimecodeService',
  'easyscott.WebsocketService',
  // 'easyscott.acn',
  // 'easyscott.devices',
  'easyscott.clips',
  'easyscott.live',
  'easyscott.slate',
  'easyscott.clink',
]);

easyscott.directive('loading', function() {
  return {
    replace: true,
    restrict: 'E',
    templateUrl: 'partials/loading.html'
  };
});

easyscott.config(function($stateProvider, $urlRouterProvider, localStorageServiceProvider) {
  // For any unmatched url, redirect to /devices
  $urlRouterProvider.otherwise("/clips");

  $stateProvider
    .state('clips', {
      url: "/clips?device",
      templateUrl: "partials/clips.html",
      controller: 'ClipsController'
    })
    .state('live', {
      url: "/live",
      templateUrl: "partials/live.html",
      controller: 'LiveController'
    })
    .state('slate', {
      url: "/slate",
      templateUrl: "partials/slate.html",
      controller: 'SlateController'
    })
    .state('clink', {
      url: "/remote",
      templateUrl: "partials/clink.html",
      controller: 'CLinkController',
      onExit: function($rootScope, WebsocketService, ChannelModel) {
        window.clearInterval($rootScope.ensureMeterTimer);
        var channelId = ChannelModel.getActiveCLinkChannelId();
        if (channelId) {
          WebsocketService.sendMessage({
            request: 'CHANNEL_COMMAND',
            id: channelId,
            command: 'METER_MONITOR',
            arguments: {
              enabled: false
            }
          });
        }
      }
    });

  // configure local storage
  localStorageServiceProvider
    .setPrefix('masterlockit')
    .setStorageType('localStorage')
    .setStorageCookie(90, '/');
});

easyscott.run(function($rootScope, $state, WebsocketService, ChannelModel, SystemPropertiesModel, ClipModel, LiveClipModel, DeviceModel, SlateModel, TimecodeService, CjamSettingsService, editableOptions) {
  jQuery('#hubMode').bootstrapSwitch({
    onText: 'On',
    onColor: 'default',
    offText: 'Off',
    onSwitchChange: function() {
      $rootScope.setEstimatedReconnectTimestamp(new Date().valueOf() + 20 * 1000);
      $rootScope.switchGeneratorMode();
      $rootScope.changingMode = true;
    }
  });

  editableOptions.theme = 'bs3';

  $rootScope.$state = $state;
  $rootScope.alerts = [];
  $rootScope.channels = [];
  $rootScope.customActions = [];
  $rootScope.estimatedReconnectTimestamp = 0;
  $rootScope.remainingSecondsUntilReconnect = 0;

  $rootScope.setActions = function(actions) {
    $rootScope.customActions = actions;
  };

  $rootScope.connected = false;
  $rootScope.changingMode = false;
  $rootScope.fullscreen = false;
  $rootScope.setFullscreen = function(value) {
    $rootScope.fullscreen = value;
  };
  $rootScope.tc = {
    seconds: 0,
    frames: 0,
    iFps: 25,
    skew: 0,
    reference: new Date().valueOf(),
    latency: 10000,
    correction: 0
  };
  $rootScope.shownClap = null;
  $rootScope.shownClapTimestamp = null;
  $rootScope.nanosRef = null;

  WebsocketService.addEventListener('connect', function() {
    $rootScope.connected = true;
  });
  WebsocketService.addEventListener('disconnect', function() {
    $rootScope.connected = false;
  });
  WebsocketService.addEventListener('message', function(type, message) {
    if (message.request === 'ACN_MODE_CHANGE') {
      $rootScope.changingMode = true;
    } else if (message.request === 'ACN_STARTUP') {
      $rootScope.generator = message.channelState.configuration.generatorMode === 'true';
      $rootScope.changingMode = false;
    } else if (message.request === 'TIMECODE') {
      $rootScope.nanosRef = {
        nanos: message.nanos,
        now: new Date().valueOf()
      }
    }
  });
  ChannelModel.addEventListener('change', function() {
    $rootScope.generator = ChannelModel.isACNGenerator();
    $rootScope.channels = ChannelModel.channels;
  });
  SystemPropertiesModel.addEventListener('change', function() {
    $rootScope.systemproperties = SystemPropertiesModel.properties;
  });

  $rootScope.restartAcn = function() {
    var channelId = ChannelModel.getActiveACNChannelId();

    if (channelId) {
      WebsocketService.sendMessage({
        request: 'CHANNEL_COMMAND',
        id: channelId,
        command: 'RESTART'
      });
    }
  };

  $rootScope.switchGeneratorMode = function() {
    var channelId = ChannelModel.getActiveACNChannelId();

    if (channelId) {
      WebsocketService.sendMessage({
        request: 'CHANNEL_COMMAND',
        id: channelId,
        command: 'CHANGE_MODE'
      });
    }
  };

  $rootScope.executeSynchronousRequest = function(message, resultCallback) {
    jQuery.ajax({
      type: "POST",
      url: "/syncapi/",
      data: JSON.stringify(message),
      contentType: "application/json; charset=utf-8",
      dataType: "json",
      success: function(data) {
        if (typeof resultCallback === "function") {
          resultCallback(data);
        }
      },
      error: function() {
        if (typeof resultCallback === "function") {
          resultCallback(null);
        }
      },
      complete: function() {
        $rootScope.$apply();
      }
    });
  };

  $rootScope.addAlert = function(type, message) {
    $rootScope.alerts.push({ type: type, message: message });
  };

  $rootScope.clearAlerts = function() {
    $rootScope.alerts = [];
  };

  $rootScope.reboot = function() {
    if (confirm("Do you really want to reboot?")) {
      WebsocketService.sendMessage('{"request":"REBOOT"}');
    }
  };

  $rootScope.shutdown = function() {
    if (confirm("Do you really want to shutdown?")) {
      WebsocketService.sendMessage('{"request":"SHUTDOWN"}');
    }
  };

  $rootScope.reset = function() {
    if (confirm("Do you really want to reset MasterLockit? This will delete all configuration and clip data from system! System will also reboot!")) {
      WebsocketService.sendMessage('{"request":"RESET"}');
    }
  };

  $rootScope.setEstimatedReconnectTimestamp = function(ts) {
    $rootScope.estimatedReconnectTimestamp = ts;
  };

  $rootScope.inDevMode = function() {
    return $rootScope.systemproperties && $rootScope.systemproperties['devmode'] === 'true';
  };

  $rootScope.isMasterLockitPlus = function() {
    return $rootScope.systemproperties && $rootScope.systemproperties['provisioning'] === 'MASTER_LOCKIT_PLUS';
  };

  $rootScope.isTcEditAllowed = function() {
    if (!$rootScope.generator) {
      return true;
    } else if (DeviceModel.getMasterclockDevice()) {
      return DeviceModel.getMasterclockDevice().properties['acnDisplayMode'] !== 'CJAM_SLAVE';
    } else {
      return false;
    }
  };

  $rootScope.requestFullscreen = function() {
    if (document.body.requestFullscreen) {
      document.body.requestFullscreen();
    } else if (document.body.webkitRequestFullscreen) {
      document.body.webkitRequestFullscreen();
    }
  };

  $rootScope.currentTc = function() {
    var fps = $rootScope.tc.iFps;
    var totalMillis = tcToMillis($rootScope.tc);
    if (!fps || !totalMillis) { return null; }

    var totalFrames = totalMillis * fps / 1000;
    var totalSeconds = totalFrames / fps;
    var hours = Math.floor((totalSeconds / 3600) % 24);
    var minutes = Math.floor((totalSeconds / 60) % 60);
    var seconds = Math.floor(totalSeconds % 60);
    var frames = Math.round(totalFrames % fps);

    return { hours: hours, minutes: minutes, seconds: seconds, frames: frames };
  };

  $rootScope.showClap = function(clap) {
    $rootScope.shownClap = clap;
    $rootScope.shownClapTimestamp = new Date().valueOf();
  };

  $rootScope.nanoDiff = function(nanos) {
    if (!$rootScope.nanosRef) { return null; }
    var localDiff = new Date().valueOf() - $rootScope.nanosRef.now;
    return ($rootScope.nanosRef.nanos + localDiff * 1000000) - nanos;
  };

  $rootScope.baseFps = baseFps;
  $rootScope.projectFpsBase = projectFpsBase;

  DeviceModel.addEventListener('change', function() {
    $rootScope.localLockit = DeviceModel.getMasterclockDevice();
    $rootScope.$apply();
  });

  ClipModel.addEventListener('clipChanged', function(type, clipType, id, clip) {
    if (clipType === 'slateClap') {
      $rootScope.showClap(clip);
    }
  });

  ChannelModel.start();
  ClipModel.start();
  LiveClipModel.start();
  DeviceModel.start();
  SystemPropertiesModel.start();
  WebsocketService.start();
  SlateModel.start();

  function projectFpsBase() {
    if ($rootScope.localLockit && $rootScope.localLockit.properties && $rootScope.localLockit.properties.projectFramerate) {
      var mapping = {
        'FPS_23_98': 24,
        'FPS_24': 24,
        'FPS_25': 25,
        'FPS_29_97': 30,
        'FPS_29_97_DROP': 30,
        'FPS_30': 30,
        'FPS_30_DROP': 30,
        'FPS_47_95': 48,
        'FPS_48': 48,
        'FPS_50': 50,
        'FPS_59_94': 60,
        'FPS_59_94_DROP': 60,
        'FPS_60': 60,
        'FPS_60_DROP': 60
      };
      return mapping[$rootScope.localLockit.properties.projectFramerate];
    } else {
      return null;
    }
  }

  function baseFps() {
    const projectFps = projectFpsBase();
    if (projectFps) {
      if (projectFps > 30) {
        return projectFps / 2;
      } else {
        return projectFps;
      }
    } else {
      return null;
    }
  }

  function ntpPulse(next) {
    var start = new Date();
    jQuery.ajax({
      type: 'POST',
      url: '/timecode'
    }).success(function(data) {
      var tc = JSON.parse(data);
      var end = new Date();
      var latency = end.valueOf() - start.valueOf();
      next({
        seconds: tc.seconds,
        frames: tc.frames,
        skew: tc.skew,
        iFps: tc.iFps,
        reference: new Date().valueOf(),
        latency: latency
      });
    }).error(function() {
      next(null);
    });
  }

  function syncTc(noSamples, next) {
    var samples = [];
    for (var i = 0; i < noSamples; i++) {
      ntpPulse(function(tc) {
        samples.push(tc);
        if (samples.length === noSamples) {
          var bestTc = _.chain(samples).filter(function(tc) { return tc; }).min(function(tc) { return tc.latency }).value()
          next(bestTc);
        }
      })
    }
  }

  function updateTc(next) {
    syncTc(3, function(tc) {
      if (!tc) {
        if (next) { next(); }
        return;
      }

      tc.correction = tcToMillis(tc) - tcToMillis($rootScope.tc);
      var delta = Math.abs(tc.correction);
      var lastTcDelta = Math.abs(new Date().valueOf() - $rootScope.tc.reference);

      if (delta >= 2000 || tc.latency <= $rootScope.tc.latency || lastTcDelta > 10000) {
        $rootScope.tc = tc;
        $rootScope.$apply();
      }

      if (delta >= 2000) {
        animateTimecode();
      }

      if (next) { next(); }
    });
  }

  function tcToMillis(tc) {
    var fps = tc.iFps;
    if (!fps) { return null; }

    var skew = new Date().valueOf() - tc.reference + tc.skew + tc.latency / 2;
    return tc.seconds * 1000 + tc.frames * (1000 / fps) + skew;
  }

  function currentTcString() {
    const tc = $rootScope.currentTc();
    if (tc) {
      return TimecodeService.render(tc);
    } else {
      return "...";
    }
  }

  $rootScope.currentTcMillis = function() {
    return tcToMillis($rootScope.tc);
  };

  $rootScope.currentTcString = currentTcString.bind(this);

  // Animate the timecode element if a new timecode was set.
  function animateTimecode() {
    var $timecodeElement = $(".running-timecode");
    var originalMargin = $timecodeElement.css("margin-top");
    $timecodeElement.animate({
      "font-size": "3.0em",
      "margin-top": "-6px"
    }, {
      duration: 500,
      complete: function() {
        $(this).animate({
          "font-size": "1.6em",
          "margin-top": originalMargin
        }, {
          duration: 500
        });
      }
    });
  }

  function loopUpdateTc() {
    updateTc(function() {
      setTimeout(function() { loopUpdateTc(); }, 2000);
    });
  }

  loopUpdateTc();

  setInterval(function() {
    // Reset clap TC
    if ($rootScope.shownClap) {
      if (!$rootScope.shownClapTimestamp || new Date().valueOf() - $rootScope.shownClapTimestamp > 2000) {
        $rootScope.shownClap = null;
        $rootScope.shownClapTimestamp = null;
      }
    }

    if ($rootScope.tcElement) {
      $rootScope.tcElement.text(currentTcString() || "...");
    } else {
      $rootScope.tcElement = $("#the-timecode");
    }
  }, 0);

  // Check/Update MasterLockit system time
  var systemTime = new Date().getTime();
  var localDateTime = new Date().toISOString();
  WebsocketService.sendMessage({
    request: 'SYSTEMTIME',
    time: systemTime,
    localDateTime: localDateTime
  });
});

