var easyscott = angular.module('easyscott', [
  'ui.router',
  'ngAnimate',
  'xeditable',
  'easyscott.filters',
  'easyscott.HelperService',
  'easyscott.WebsocketService',
  'easyscott.update',
]);

easyscott.config(function($stateProvider, $urlRouterProvider) {
  // For any unmatched url, redirect to /devices
  $urlRouterProvider.otherwise("/update");

  $stateProvider
    .state('update', {
      url: "/update",
      templateUrl: "partials/migration.html",
      controller: 'UpdateController',
    })
});

easyscott.run(function($rootScope, $state, WebsocketService, editableOptions) {
  editableOptions.theme = 'bs3';

  $rootScope.$state = $state;
  $rootScope.alerts = [];
  $rootScope.connected = false;
  $rootScope.estimatedReconnectTimestamp = 0;
  $rootScope.remainingSecondsUntilReconnect = 0;
  
  WebsocketService.addEventListener('connect', function() {
    $rootScope.connected = true;
    $rootScope.$apply();
  });
  WebsocketService.addEventListener('disconnect', function() {
    $rootScope.connected = false;
    $rootScope.$apply();
  });

  $rootScope.addAlert = function(type, message) {
    $rootScope.alerts.push({ type: type, message: message});
  };

  $rootScope.clearAlerts = function() {
    $rootScope.alerts = [];
  };

  $rootScope.reboot = function() {
    if(confirm("Do you really want to reboot?")) {
      WebsocketService.sendMessage('{"request":"REBOOT"}');
    }
  };

  $rootScope.shutdown = function() {
    if(confirm("Do you really want to shutdown?")) {
      WebsocketService.sendMessage('{"request":"SHUTDOWN"}');
    }
  };

  $rootScope.reset = function() {
    if(confirm("Do you really want to reset MasterLockit? This will delete all configuration and clip data from system! System will also reboot!")) {
      WebsocketService.sendMessage('{"request":"RESET"}');
    }
  };

  $rootScope.setEstimatedReconnectTimestamp = function(ts) {
    $rootScope.estimatedReconnectTimestamp = ts;
  }

  setInterval(function() {
    var reconnectDiff = $rootScope.estimatedReconnectTimestamp - new Date();
    $rootScope.remainingSecondsUntilReconnect = Math.floor(Math.max(0, reconnectDiff / 1000));
    $rootScope.$apply();
  }, 500);

  
  WebsocketService.start();
});
